import unittest
from bibtexparser.bparser import BibTexParser
from bibtexparser.bwriter import to_bibtex


""" The code is supposed to treat comments the following way:
    Each @Comment opens a comment that ends when something
    that is not a comment is encountered. More precisely
    this means a line starting with an @. Lines that are not
    parsed as anything else are also considered comments.
    If the comment starts and ends with braces, they are removed.

    Current issues:
        - a comment followed by a line starting with @smthing
        that is not a valid bibtex element are parsed separately,
        that is as two comments.
        - braces are either ignored or removed which is not easily
        predictable.
"""


class TestParseComment(unittest.TestCase):

    def test_comment_count(self):
        with open('bibtexparser/tests/data/features.bib') as bibfile:
            bib = BibTexParser(bibfile.read())
        self.assertEqual(len(bib.comments), 3)

    def test_comment_list(self):
        with open('bibtexparser/tests/data/features.bib') as bibfile:
            bib = BibTexParser(bibfile.read())
        expected = ["ignore this line!",
                    "ignore this line too!",
                    "and ignore this line too!"]
        self.assertEqual(bib.comments, expected)

    def test_multiline_comments(self):
        with open('bibtexparser/tests/data/multiline_comments.bib') as bibfile:
            bib = BibTexParser(bibfile.read())
        expected = [
"""Lorem ipsum dolor sit amet,
consectetur adipisicing elit""",
"""
Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.

Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur.
Excepteur sint occaecat cupidatat non proident.
 ,
""",
"""


Sunt in culpa qui officia deserunt mollit anim id est laborum.


""",
""
        ]
        self.maxDiff = None
        self.assertEqual(bib.comments, expected)

    def test_multiple_entries(self):
        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibfile:
            bparser = BibTexParser()
            bib = bparser.parse_file(bibfile)
        expected = ["",
                    "A comment"]
        self.assertEqual(bib.comments, expected)

    def test_comments_percentage(self):
        with open('bibtexparser/tests/data/comments_percentage.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean Cesar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Baltazar2013',
                     'year': '2013',
                     'author': 'Jean Baltazar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_comments_percentage_nocoma(self):
        with open('bibtexparser/tests/data/comments_percentage_nolastcoma.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean Cesar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Baltazar2013',
                     'year': '2013',
                     'author': 'Jean Baltazar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_no_newline(self):
        comments = """This is a comment."""
        expected = ["This is a comment."]
        bib = BibTexParser(comments)
        self.assertEqual(bib.comments, expected)

    def test_43(self):
        comment = "@STRING{foo = \"bar\"}\n" \
                  "This is a comment\n" \
                  "This is a second comment."
        expected = "This is a comment\nThis is a second comment."
        bib = BibTexParser(comment, common_strings=False)
        self.assertEqual(bib.comments, [expected])
        self.assertEqual(bib.strings, {'foo': 'bar'})

    def test_43_bis(self):
        comment = "@STRING{foo = \"bar\"}\n" \
                  "This is a comment\n" \
                  "STRING{Baz = \"This should be interpreted as comment.\"}"
        expected = "This is a comment\n" \
                   "STRING{Baz = \"This should be interpreted as comment.\"}"
        bib = BibTexParser(comment, common_strings=False)
        self.assertEqual(bib.comments, [expected])
        self.assertEqual(bib.strings, {'foo': 'bar'})


class TestWriteComment(unittest.TestCase):
    def test_comment_write(self):
        with open('bibtexparser/tests/data/comments_only.bib') as bibfile:
            bib = BibTexParser(bibfile.read())

        with open('bibtexparser/tests/data/comments_only_output.bib') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.assertEqual(result, expected)

    def test_multiline_comment_write(self):
        with open('bibtexparser/tests/data/multiline_comments.bib') as bibfile:
            expected = bibfile.read()

        bib = BibTexParser(expected)
        result = to_bibtex(bib)
        self.assertEqual(result, expected)

    def test_multiple_entries(self):
        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibfile:
            bib = BibTexParser(bibfile.read())
        with open('bibtexparser/tests/data/multiple_entries_and_comments_output.bib') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.assertEqual(result, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import unittest

from bibtexparser.customization import getnames, convert_to_unicode, homogenize_latex_encoding, page_double_hyphen, keyword, add_plaintext_fields


class TestBibtexParserMethod(unittest.TestCase):

    ###########
    # getnames
    ###########
    def test_getnames(self):
        names = ['Foo Bar',
                 'Foo B. Bar',
                 'F. B. Bar',
                 'F.B. Bar',
                 'F. Bar',
                 'Jean de Savigny',
                 'Jean la Tour',
                 'Jean le Tour',
                 'Mike ben Akar',
                 'A. {Delgado de Molina}',
                 r'M. Vign{\'e}',
                 'Tom {de Geus}',
                 'Tom {de \{Geus}',
                 'Tom \{de Geus\}',
                 'Tom de {G\{eus}',
                 'Foo B{\'a}r',
                 r'{G{\'{e}}rard} {Ben Arous}',
                 'Incorrect {{name}',
                 #'Jean de la Tour',
                 #'Johannes Diderik van der Waals',
                 ]
        result = getnames(names)
        expected = ['Bar, Foo',
                    'Bar, Foo B.',
                    'Bar, F. B.',
                    'Bar, F. B.',
                    'Bar, F.',
                    'de Savigny, Jean',
                    'la Tour, Jean',
                    'le Tour, Jean',
                    'ben Akar, Mike',
                    '{Delgado de Molina}, A.',
                    r'Vign{\'e}, M.',
                    '{de Geus}, Tom',
                    '{de \{Geus}, Tom',
                    'Geus\}, Tom \{de',
                    'de {G\{eus}, Tom',
                    'B{\'a}r, Foo',
                    r'{Ben Arous}, {G{\'{e}}rard}',
                    'Incorrect {{name}',
                    #'de la Tour, Jean',
                    #'van der Waals, Johannes Diderik',
                    ]
        self.assertEqual(result, expected)

    ###########
    # page_double_hyphen
    ###########
    def test_page_double_hyphen_alreadyOK(self):
        record = {'pages': '12--24'}
        result = page_double_hyphen(record)
        expected = record
        self.assertEqual(result, expected)

    def test_page_double_hyphen_simple(self):
        record = {'pages': '12-24'}
        result = page_double_hyphen(record)
        expected = {'pages': '12--24'}
        self.assertEqual(result, expected)

    def test_page_double_hyphen_space(self):
        record = {'pages': '12 - 24'}
        result = page_double_hyphen(record)
        expected = {'pages': '12--24'}
        self.assertEqual(result, expected)

    def test_page_double_hyphen_nothing(self):
        record = {'pages': '12 24'}
        result = page_double_hyphen(record)
        expected = {'pages': '12 24'}
        self.assertEqual(result, expected)

    ###########
    # convert to unicode
    ###########
    def test_convert_to_unicode(self):
        record = {'toto': r'{\`a} \`{a}'}
        result = convert_to_unicode(record)
        expected = {'toto': r'à à'}
        self.assertEqual(result, expected)
        record = {'toto': r'{\"u} \"{u}'}
        result = convert_to_unicode(record)
        expected = {'toto': r'ü ü'}
        self.assertEqual(result, expected)
        # From issue 121
        record = {'title': r'{Two Gedenk\"uberlieferung der Angelsachsen}'}
        result = convert_to_unicode(record)
        expected = {'title': r'Two Gedenküberlieferung der Angelsachsen'}
        self.assertEqual(result, expected)
        # From issue 161
        record = {'title': r"p\^{a}t\'{e}"}
        result = convert_to_unicode(record)
        expected = {'title': r"pâté"}
        self.assertEqual(result, expected)
        record = {'title': r"\^{i}le"}
        result = convert_to_unicode(record)
        expected = {'title': r"île"}
        self.assertEqual(result, expected)
        record = {'title': r"\texttimes{}{\texttimes}\texttimes"}
        result = convert_to_unicode(record)
        expected = {'title': r"×××"}
        self.assertEqual(result, expected)

    ###########
    # homogenize
    ###########
    def test_homogenize(self):
        record = {'toto': r'à {\`a} \`{a}'}
        result = homogenize_latex_encoding(record)
        expected = {'toto': r'{\`a} {\`a} {\`a}'}
        self.assertEqual(result, expected)

    ###########
    # add_plaintext_fields
    ###########
    def test_add_plaintext_fields(self):
        record = {
            'title': r'On-line {Recognition} of {Handwritten} {Mathematical} {Symbols}',
            'foobar': ['{FFT} {Foobar}', '{foobar}'],
            'foobar2': {'item1': '{FFT} {Foobar}', 'item2': '{foobar}'}
        }
        result = add_plaintext_fields(record)
        expected = {
            'title': 'On-line {Recognition} of {Handwritten} {Mathematical} {Symbols}',
            'plain_title': 'On-line Recognition of Handwritten Mathematical Symbols',
            'foobar': ['{FFT} {Foobar}', '{foobar}'],
            'plain_foobar': ['FFT Foobar', 'foobar'],
            'foobar2': {'item1': '{FFT} {Foobar}', 'item2': '{foobar}'},
            'plain_foobar2': {'item1': 'FFT Foobar', 'item2': 'foobar'}
        }
        self.assertEqual(result, expected)

    ###########
    # keywords
    ###########
    def test_keywords(self):
        record = {'keyword': "a b, a b , a b;a b ; a b, a b\n"}
        result = keyword(record)
        expected = {'keyword': ['a b'] * 6}
        self.assertEqual(result, expected)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from bibtexparser.customization import InvalidName, splitname

class TestSplitnameMethod(unittest.TestCase):
    def test_splitname_basic(self):
        """Basic tests of customization.splitname() """
        # Empty input.
        result = splitname("")
        expected = {}
        self.assertEqual(result, expected, msg="Invalid output for empty name")

        # Non-whitespace names.
        result = splitname("    ")
        expected = {}
        self.assertEqual(result, expected, msg="Invalid output for space-only name")
        result = splitname("  \t~~")
        expected = {}
        self.assertEqual(result, expected, msg="Invalid output for whitespace name")

        # Test strict mode.
        with self.assertRaises(InvalidName):         # Trailing comma (4 cases).
            splitname("BB,", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("BB,  ", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("BB, ~\t", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname(", ~\t", strict_mode=True)
        with self.assertRaises(InvalidName):         # Too many sections.
            splitname("AA, BB, CC, DD", strict_mode=True)
        with self.assertRaises(InvalidName):         # Unterminated opening brace (x3).
            splitname("AA {BB CC", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("AA {{{BB CC", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("AA {{{BB} CC}", strict_mode=True)
        with self.assertRaises(InvalidName):         # Unmatched closing brace (x3).
            splitname("AA BB CC}", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("AA BB CC}}}", strict_mode=True)
        with self.assertRaises(InvalidName):
            splitname("{AA {BB CC}}}", strict_mode=True)

        # Test strict mode off for trailing comma.
        expected = {'first': [], 'von': [], 'last': ["BB"], 'jr': []}
        result = splitname("BB,", strict_mode=False)
        self.assertEqual(result, expected, msg="Invalid output for trailing comma with strict mode off")
        result = splitname("BB,   ", strict_mode=False)
        self.assertEqual(result, expected, msg="Invalid output for trailing comma with strict mode off")
        result = splitname("BB,  ~\t ", strict_mode=False)
        self.assertEqual(result, expected, msg="Invalid output for trailing comma with strict mode off")
        expected = {}
        result = splitname(",  ~\t", strict_mode=False)
        self.assertEqual(result, expected, msg="Invalid output for trailing comma with strict mode off")

        # Test strict mode off for too many sections.
        expected = {'first': ["CC", "DD"], 'von': [], 'last': ["AA"], 'jr': ["BB"]}
        result = splitname("AA, BB, CC, DD", strict_mode=False)
        self.assertEqual(result, expected, msg="Invalid output for too many sections with strict mode off")

        # Test strict mode off for an unterminated opening brace.
        result = splitname("AA {BB CC", strict_mode=False)
        expected = {'first': ["AA"], 'von': [], 'last': ["{BB CC}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unterminated opening brace with strict mode off")
        result = splitname("AA {{{BB CC", strict_mode=False)
        expected = {'first': ["AA"], 'von': [], 'last': ["{{{BB CC}}}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unterminated opening brace with strict mode off")
        result = splitname("AA {{{BB} CC}", strict_mode=False)
        expected = {'first': ["AA"], 'von': [], 'last': ["{{{BB} CC}}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unterminated opening brace with strict mode off")

        # Test strict mode off for an unmatched closing brace.
        result = splitname("AA BB CC}", strict_mode=False)
        expected = {'first': ["AA", "BB"], 'von': [], 'last': ["{CC}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unmatched closing brace with strict mode off")
        result = splitname("AA BB CC}}}", strict_mode=False)
        expected = {'first': ["AA", "BB"], 'von': [], 'last': ["{{{CC}}}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unmatched closing brace with strict mode off")
        result = splitname("{AA {BB CC}}}", strict_mode=False)
        expected = {'first': [], 'von': [], 'last': ["{{AA {BB CC}}}"], 'jr': []}
        self.assertEqual(result, expected, msg="Invalid output for unmatched closing brace with strict mode off")

        # Test it handles commas at higher brace levels.
        result = splitname("CC, dd, {AA, BB}")
        expected = {'first': ["{AA, BB}"], 'von': [], 'last': ["CC"], 'jr': ["dd"]}
        self.assertEqual(result, expected, msg="Invalid output for braced commas")


    def test_splitname_cases(self):
        """Test customization.splitname() vs output from BibTeX """
        for name, expected in splitname_test_cases:
            result = splitname(name)
            self.assertEqual(result, expected, msg="Input name: {0}".format(name))


splitname_test_cases = (
    (r'Per Brinch Hansen',
     {'first': ['Per', 'Brinch'], 'von': [], 'last': ['Hansen'], 'jr': []}),

    (r'Brinch Hansen, Per',
     {'first': ['Per'], 'von': [], 'last': ['Brinch', 'Hansen'], 'jr': []}),

    (r'Brinch Hansen,, Per',
     {'first': ['Per'], 'von': [], 'last': ['Brinch', 'Hansen'], 'jr': []}),

    (r"Charles Louis Xavier Joseph de la Vall{\'e}e Poussin",
     {'first': ['Charles', 'Louis', 'Xavier', 'Joseph'], 'von': ['de', 'la'],
      'last':  [r'Vall{\'e}e', 'Poussin'], 'jr': []}),

    (r'D[onald] E. Knuth',
     {'first': ['D[onald]', 'E.'], 'von': [], 'last': ['Knuth'], 'jr': []}),

    (r'A. {Delgado de Molina}',
     {'first': ['A.'], 'von': [], 'last': ['{Delgado de Molina}'], 'jr': []}),

    (r"M. Vign{\'e}",
     {'first': ['M.'], 'von': [], 'last': [r"Vign{\'e}"], 'jr': []}),

###############################################################################
#
# Test cases from
# http://maverick.inria.fr/~Xavier.Decoret/resources/xdkbibtex/bibtex_summary.html
#
###############################################################################

    (r'AA BB',
     {'first': ['AA'], 'von': [], 'last': ['BB'], 'jr': []}),

    (r'AA',
     {'first': [], 'von': [], 'last': ['AA'], 'jr': []}),

    (r'AA bb',
     {'first': ['AA'], 'von': [], 'last': ['bb'], 'jr': []}),

    (r'aa',
     {'first': [], 'von': [], 'last': ['aa'], 'jr': []}),

    (r'AA bb CC',
     {'first': ['AA'], 'von': ['bb'], 'last': ['CC'], 'jr': []}),

    (r'AA bb CC dd EE',
     {'first': ['AA'], 'von': ['bb', 'CC', 'dd'], 'last': ['EE'], 'jr': []}),

    (r'AA 1B cc dd',
     {'first': ['AA', '1B'], 'von': ['cc'], 'last': ['dd'], 'jr': []}),

    (r'AA 1b cc dd',
     {'first': ['AA'], 'von': ['1b', 'cc'], 'last': ['dd'], 'jr': []}),

    (r'AA {b}B cc dd',
     {'first': ['AA', '{b}B'], 'von': ['cc'], 'last': ['dd'], 'jr': []}),

    (r'AA {b}b cc dd',
     {'first': ['AA'], 'von': ['{b}b', 'cc'], 'last': ['dd'], 'jr': []}),

    (r'AA {B}b cc dd',
     {'first': ['AA'], 'von': ['{B}b', 'cc'], 'last': ['dd'], 'jr': []}),

    (r'AA {B}B cc dd',
     {'first': ['AA', '{B}B'], 'von': ['cc'], 'last': ['dd'], 'jr': []}),

    (r'AA \BB{b} cc dd',
     {'first': ['AA', r'\BB{b}'], 'von': ['cc'], 'last': ['dd'], 'jr': []}),

    (r'AA \bb{b} cc dd',
     {'first': ['AA'], 'von': [r'\bb{b}', 'cc'], 'last': ['dd'], 'jr': []}),

    (r'AA {bb} cc DD',
     {'first': ['AA', '{bb}'], 'von': ['cc'], 'last': ['DD'], 'jr': []}),

    (r'AA bb {cc} DD',
     {'first': ['AA'], 'von': ['bb'], 'last': ['{cc}', 'DD'], 'jr': []}),

    (r'AA {bb} CC',
     {'first': ['AA', '{bb}'], 'von': [], 'last': ['CC'], 'jr': []}),

    (r'bb CC, AA',
     {'first': ['AA'], 'von': ['bb'], 'last': ['CC'], 'jr': []}),

    (r'bb CC, aa',
     {'first': ['aa'], 'von': ['bb'], 'last': ['CC'], 'jr': []}),

    (r'bb CC dd EE, AA',
     {'first': ['AA'], 'von': ['bb', 'CC', 'dd'], 'last': ['EE'], 'jr': []}),

    (r'bb, AA',
     {'first': ['AA'], 'von': [], 'last': ['bb'], 'jr': []}),

    (r'bb CC,XX, AA',
     {'first': ['AA'], 'von': ['bb'], 'last': ['CC'], 'jr': ['XX']}),

    (r'bb CC,xx, AA',
     {'first': ['AA'], 'von': ['bb'], 'last': ['CC'], 'jr': ['xx']}),

    (r'BB,, AA',
     {'first': ['AA'], 'von': [], 'last': ['BB'], 'jr': []}),

    (r"Paul \'Emile Victor",
     {'first': ['Paul', r"\'Emile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r"Paul {\'E}mile Victor",
     {'first': ['Paul', r"{\'E}mile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r"Paul \'emile Victor",
     {'first': ['Paul'], 'von': [r"\'emile"], 'last': ['Victor'], 'jr': []}),

    (r"Paul {\'e}mile Victor",
     {'first': ['Paul'], 'von': [r"{\'e}mile"], 'last': ['Victor'], 'jr': []}),

    (r"Victor, Paul \'Emile",
     {'first': ['Paul', r"\'Emile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r"Victor, Paul {\'E}mile",
     {'first': ['Paul', r"{\'E}mile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r"Victor, Paul \'emile",
     {'first': ['Paul', r"\'emile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r"Victor, Paul {\'e}mile",
     {'first': ['Paul', r"{\'e}mile"], 'von': [], 'last': ['Victor'], 'jr': []}),

    (r'Dominique Galouzeau de Villepin',
     {'first': ['Dominique', 'Galouzeau'], 'von': ['de'], 'last': ['Villepin'], 'jr': []}),

    (r'Dominique {G}alouzeau de Villepin',
     {'first': ['Dominique'], 'von': ['{G}alouzeau', 'de'],
      'last': ['Villepin'], 'jr': []}),

    (r'Galouzeau de Villepin, Dominique',
     {'first': ['Dominique'], 'von': ['Galouzeau', 'de'],
      'last': ['Villepin'], 'jr': []}),

###############################################################################
#
# Test cases from pybtex
# See file /pybtex/tests/parse_name_test.py in the pybtex source.
#
###############################################################################

    (r'A. E.                   Siegman',
     {'first': ['A.', 'E.'], 'von': [], 'last': ['Siegman'], 'jr': []}),

    (r'A. G. W. Cameron',
     {'first': ['A.', 'G.', 'W.'], 'von': [], 'last': ['Cameron'], 'jr': []}),

    (r'A. Hoenig',
     {'first': ['A.'], 'von': [], 'last': ['Hoenig'], 'jr': []}),

    (r'A. J. Van Haagen',
     {'first': ['A.', 'J.', 'Van'], 'von': [], 'last': ['Haagen'], 'jr': []}),

    (r'A. S. Berdnikov',
     {'first': ['A.', 'S.'], 'von': [], 'last': ['Berdnikov'], 'jr': []}),

    (r'A. Trevorrow',
     {'first': ['A.'], 'von': [], 'last': ['Trevorrow'], 'jr': []}),

    (r'Adam H. Lewenberg',
     {'first': ['Adam', 'H.'], 'von': [], 'last': ['Lewenberg'], 'jr': []}),

    (r'Addison-Wesley Publishing Company',
     {'first': ['Addison-Wesley', 'Publishing'], 'von': [],
      'last': ['Company'], 'jr': []}),

    (r'Advogato (Raph Levien)',
     {'first': ['Advogato', '(Raph'], 'von': [], 'last': ['Levien)'], 'jr': []}),

    (r'Andrea de Leeuw van Weenen',
     {'first': ['Andrea'], 'von': ['de', 'Leeuw', 'van'], 'last': ['Weenen'], 'jr': []}),

    (r'Andreas Geyer-Schulz',
     {'first': ['Andreas'], 'von': [], 'last': ['Geyer-Schulz'], 'jr': []}),

    (r'Andr{\'e} Heck',
     {'first': [r'Andr{\'e}'], 'von': [], 'last': ['Heck'], 'jr': []}),

    (r'Anne Br{\"u}ggemann-Klein',
     {'first': ['Anne'], 'von': [], 'last': [r'Br{\"u}ggemann-Klein'], 'jr': []}),

    (r'Anonymous',
     {'first': [], 'von': [], 'last': ['Anonymous'], 'jr': []}),

    (r'B. Beeton',
     {'first': ['B.'], 'von': [], 'last': ['Beeton'], 'jr': []}),

    (r'B. Hamilton Kelly',
     {'first': ['B.', 'Hamilton'], 'von': [], 'last': ['Kelly'], 'jr': []}),

    (r'B. V. Venkata Krishna Sastry',
     {'first': ['B.', 'V.', 'Venkata', 'Krishna'], 'von': [],
      'last': ['Sastry'], 'jr': []}),

    (r'Benedict L{\o}fstedt',
     {'first': ['Benedict'], 'von': [], 'last': [r'L{\o}fstedt'], 'jr': []}),

    (r'Bogus{\l}aw Jackowski',
     {'first': ['Bogus{\l}aw'], 'von': [], 'last': ['Jackowski'], 'jr': []}),

    (r'Christina A. L.\ Thiele',
     {'first': ['Christina', 'A.', 'L.\\'], 'von': [],
      'last': ['Thiele'], 'jr': []}),

    (r"D. Men'shikov",
     {'first': ['D.'], 'von': [], 'last': ["Men'shikov"], 'jr': []}),

    (r'Darko \v{Z}ubrini{\'c}',
     {'first': ['Darko'], 'von': [], 'last': [r'\v{Z}ubrini{\'c}'], 'jr': []}),

    (r'Dunja Mladeni{\'c}',
     {'first': ['Dunja'], 'von': [], 'last': [r'Mladeni{\'c}'], 'jr': []}),

    (r'Edwin V. {Bell, II}',
     {'first': ['Edwin', 'V.'], 'von': [], 'last': ['{Bell, II}'], 'jr': []}),

    (r'Frank G. {Bennett, Jr.}',
     {'first': ['Frank', 'G.'], 'von': [], 'last': ['{Bennett, Jr.}'], 'jr': []}),

    (r'Fr{\'e}d{\'e}ric Boulanger',
     {'first': [r'Fr{\'e}d{\'e}ric'], 'von': [], 'last': ['Boulanger'], 'jr': []}),

    (r'Ford, Jr., Henry',
     {'first': ['Henry'], 'von': [], 'last': ['Ford'], 'jr': ['Jr.']}),

    (r'mr Ford, Jr., Henry',
     {'first': ['Henry'], 'von': ['mr'], 'last': ['Ford'], 'jr': ['Jr.']}),

    (r'Fukui Rei',
     {'first': ['Fukui'], 'von': [], 'last': ['Rei'], 'jr': []}),

    (r'G. Gr{\"a}tzer',
     {'first': ['G.'], 'von': [], 'last': [r'Gr{\"a}tzer'], 'jr': []}),

    (r'George Gr{\"a}tzer',
     {'first': ['George'], 'von': [], 'last': [r'Gr{\"a}tzer'], 'jr': []}),

    (r'Georgia K. M. Tobin',
     {'first': ['Georgia', 'K.', 'M.'], 'von': [], 'last': ['Tobin'], 'jr': []}),

    (r'Gilbert van den Dobbelsteen',
     {'first': ['Gilbert'], 'von': ['van', 'den'], 'last': ['Dobbelsteen'], 'jr': []}),

    (r'Gy{\"o}ngyi Bujdos{\'o}',
     {'first': [r'Gy{\"o}ngyi'], 'von': [], 'last': [r'Bujdos{\'o}'], 'jr': []}),

    (r'Helmut J{\"u}rgensen',
     {'first': ['Helmut'], 'von': [], 'last': [r'J{\"u}rgensen'], 'jr': []}),

    (r'Herbert Vo{\ss}',
     {'first': ['Herbert'], 'von': [], 'last': ['Vo{\ss}'], 'jr': []}),

    (r"H{\'a}n Th{\^e}\llap{\raise 0.5ex\hbox{\'{\relax}}} Th{\'a}nh",
     {'first': [r'H{\'a}n', r"Th{\^e}\llap{\raise 0.5ex\hbox{\'{\relax}}}"],
      'von': [], 'last': [r"Th{\'a}nh"], 'jr': []}),

    (r"H{\`a}n Th\^e\llap{\raise0.5ex\hbox{\'{\relax}}} Th{\`a}nh",
     {'first': [r'H{\`a}n', r"Th\^e\llap{\raise0.5ex\hbox{\'{\relax}}}"],
      'von': [], 'last': [r"Th{\`a}nh"], 'jr': []}),

    (r'J. Vesel{\'y}',
     {'first': ['J.'], 'von': [], 'last': [r'Vesel{\'y}'], 'jr': []}),

    (r'Javier Rodr\'{\i}guez Laguna',
     {'first': ['Javier', r'Rodr\'{\i}guez'], 'von': [], 'last': ['Laguna'], 'jr': []}),

    (r'Ji\v{r}\'{\i} Vesel{\'y}',
     {'first': [r'Ji\v{r}\'{\i}'], 'von': [], 'last': [r'Vesel{\'y}'], 'jr': []}),

    (r'Ji\v{r}\'{\i} Zlatu{\v{s}}ka',
     {'first': [r'Ji\v{r}\'{\i}'], 'von': [], 'last': [r'Zlatu{\v{s}}ka'], 'jr': []}),

    (r'Ji\v{r}{\'\i} Vesel{\'y}',
     {'first': [r'Ji\v{r}{\'\i}'], 'von': [], 'last': [r'Vesel{\'y}'], 'jr': []}),

    (r'Ji\v{r}{\'{\i}}Zlatu{\v{s}}ka',
     {'first': [], 'von': [], 'last': [r'Ji\v{r}{\'{\i}}Zlatu{\v{s}}ka'], 'jr': []}),

    (r'Jim Hef{}feron',
     {'first': ['Jim'], 'von': [], 'last': ['Hef{}feron'], 'jr': []}),

    (r'J{\"o}rg Knappen',
     {'first': [r'J{\"o}rg'], 'von': [], 'last': ['Knappen'], 'jr': []}),

    (r'J{\"o}rgen L. Pind',
     {'first': [r'J{\"o}rgen', 'L.'], 'von': [], 'last': ['Pind'], 'jr': []}),

    (r'J{\'e}r\^ome Laurens',
     {'first': [r'J{\'e}r\^ome'], 'von': [], 'last': ['Laurens'], 'jr': []}),

    (r'J{{\"o}}rg Knappen',
     {'first': [r'J{{\"o}}rg'], 'von': [], 'last': ['Knappen'], 'jr': []}),

    (r'K. Anil Kumar',
     {'first': ['K.', 'Anil'], 'von': [], 'last': ['Kumar'], 'jr': []}),

    (r'Karel Hor{\'a}k',
     {'first': ['Karel'], 'von': [], 'last': [r'Hor{\'a}k'], 'jr': []}),

    (r'Karel P\'{\i}{\v{s}}ka',
     {'first': ['Karel'], 'von': [], 'last': [r'P\'{\i}{\v{s}}ka'], 'jr': []}),

    (r'Karel P{\'\i}{\v{s}}ka',
     {'first': ['Karel'], 'von': [], 'last': [r'P{\'\i}{\v{s}}ka'], 'jr': []}),

    (r'Karel Skoup\'{y}',
     {'first': ['Karel'], 'von': [], 'last': [r'Skoup\'{y}'], 'jr': []}),

    (r'Karel Skoup{\'y}',
     {'first': ['Karel'], 'von': [], 'last': [r'Skoup{\'y}'], 'jr': []}),

    (r'Kent McPherson',
     {'first': ['Kent'], 'von': [], 'last': ['McPherson'], 'jr': []}),

    (r'Klaus H{\"o}ppner',
     {'first': ['Klaus'], 'von': [], 'last': [r'H{\"o}ppner'], 'jr': []}),

    (r'Lars Hellstr{\"o}m',
     {'first': ['Lars'], 'von': [], 'last': [r'Hellstr{\"o}m'], 'jr': []}),

    (r'Laura Elizabeth Jackson',
     {'first': ['Laura', 'Elizabeth'], 'von': [], 'last': ['Jackson'], 'jr': []}),

    (r'M. D{\'{\i}}az',
     {'first': ['M.'], 'von': [], 'last': [r'D{\'{\i}}az'], 'jr': []}),

    (r'M/iche/al /O Searc/oid',
     {'first': [r'M/iche/al', r'/O'], 'von': [], 'last': [r'Searc/oid'], 'jr': []}),

    (r'Marek Ry{\'c}ko',
     {'first': ['Marek'], 'von': [], 'last': [r'Ry{\'c}ko'], 'jr': []}),

    (r'Marina Yu. Nikulina',
     {'first': ['Marina', 'Yu.'], 'von': [], 'last': ['Nikulina'], 'jr': []}),

    (r'Max D{\'{\i}}az',
     {'first': ['Max'], 'von': [], 'last': [r'D{\'{\i}}az'], 'jr': []}),

    (r'Merry Obrecht Sawdey',
     {'first': ['Merry', 'Obrecht'], 'von': [], 'last': ['Sawdey'], 'jr': []}),

    (r'Miroslava Mis{\'a}kov{\'a}',
     {'first': ['Miroslava'], 'von': [], 'last': [r'Mis{\'a}kov{\'a}'], 'jr': []}),

    (r'N. A. F. M. Poppelier',
     {'first': ['N.', 'A.', 'F.', 'M.'], 'von': [], 'last': ['Poppelier'], 'jr': []}),

    (r'Nico A. F. M. Poppelier',
     {'first': ['Nico', 'A.', 'F.', 'M.'], 'von': [], 'last': ['Poppelier'], 'jr': []}),

    (r'Onofrio de Bari',
     {'first': ['Onofrio'], 'von': ['de'], 'last': ['Bari'], 'jr': []}),

    (r'Pablo Rosell-Gonz{\'a}lez',
     {'first': ['Pablo'], 'von': [], 'last': [r'Rosell-Gonz{\'a}lez'], 'jr': []}),

    (r'Paco La                  Bruna',
     {'first': ['Paco', 'La'], 'von': [], 'last': ['Bruna'], 'jr': []}),

    (r'Paul                  Franchi-Zannettacci',
     {'first': ['Paul'], 'von': [], 'last': ['Franchi-Zannettacci'], 'jr': []}),

    (r'Pavel \v{S}eve\v{c}ek',
     {'first': ['Pavel'], 'von': [], 'last': [r'\v{S}eve\v{c}ek'], 'jr': []}),

    (r'Petr Ol{\v{s}}ak',
     {'first': ['Petr'], 'von': [], 'last': [r'Ol{\v{s}}ak'], 'jr': []}),

    (r'Petr Ol{\v{s}}{\'a}k',
     {'first': ['Petr'], 'von': [], 'last': [r'Ol{\v{s}}{\'a}k'], 'jr': []}),

    (r'Primo\v{z} Peterlin',
     {'first': [r'Primo\v{z}'], 'von': [], 'last': ['Peterlin'], 'jr': []}),

    (r'Prof. Alban Grimm',
     {'first': ['Prof.', 'Alban'], 'von': [], 'last': ['Grimm'], 'jr': []}),

    (r'P{\'e}ter Husz{\'a}r',
     {'first': [r'P{\'e}ter'], 'von': [], 'last': [r'Husz{\'a}r'], 'jr': []}),

    (r'P{\'e}ter Szab{\'o}',
     {'first': [r'P{\'e}ter'], 'von': [], 'last': [r'Szab{\'o}'], 'jr': []}),

    (r'Rafa{\l}\.Zbikowski',
     {'first': [], 'von': [], 'last': [r'Rafa{\l}\.Zbikowski'], 'jr': []}),

    (r'Rainer Sch{\"o}pf',
     {'first': ['Rainer'], 'von': [], 'last': [r'Sch{\"o}pf'], 'jr': []}),

    (r'T. L. (Frank) Pappas',
     {'first': ['T.', 'L.', '(Frank)'], 'von': [], 'last': ['Pappas'], 'jr': []}),

    (r'TUG 2004 conference',
     {'first': ['TUG', '2004'], 'von': [], 'last': ['conference'], 'jr': []}),

    (r'TUG {\sltt DVI} Driver Standards Committee',
     {'first': ['TUG', '{\sltt DVI}', 'Driver', 'Standards'], 'von': [],
      'last': ['Committee'], 'jr': []}),

    (r'TUG {\sltt xDVIx} Driver Standards Committee',
     {'first': ['TUG'], 'von': ['{\sltt xDVIx}'],
      'last': ['Driver', 'Standards', 'Committee'], 'jr': []}),

    (r'University of M{\"u}nster',
     {'first': ['University'], 'von': ['of'], 'last': [r'M{\"u}nster'], 'jr': []}),

    (r'Walter van der Laan',
     {'first': ['Walter'], 'von': ['van', 'der'], 'last': ['Laan'], 'jr': []}),

    (r'Wendy G.                  McKay',
     {'first': ['Wendy', 'G.'], 'von': [], 'last': ['McKay'], 'jr': []}),

    (r'Wendy McKay',
     {'first': ['Wendy'], 'von': [], 'last': ['McKay'], 'jr': []}),

    (r'W{\l}odek Bzyl',
     {'first': [r'W{\l}odek'], 'von': [], 'last': ['Bzyl'], 'jr': []}),

    (r'\LaTeX Project Team',
     {'first': [r'\LaTeX', 'Project'], 'von': [], 'last': ['Team'], 'jr': []}),

    (r'\rlap{Lutz Birkhahn}',
     {'first': [], 'von': [], 'last': [r'\rlap{Lutz Birkhahn}'], 'jr': []}),

    (r'{Jim Hef{}feron}',
     {'first': [], 'von': [], 'last': ['{Jim Hef{}feron}'], 'jr': []}),

    (r'{Kristoffer H\o{}gsbro Rose}',
     {'first': [], 'von': [], 'last': ['{Kristoffer H\o{}gsbro Rose}'], 'jr': []}),

    (r'{TUG} {Working} {Group} on a {\TeX} {Directory} {Structure}',
     {'first': ['{TUG}', '{Working}', '{Group}'], 'von': ['on', 'a'],
      'last': [r'{\TeX}', '{Directory}', '{Structure}'], 'jr': []}),

    (r'{The \TUB{} Team}',
     {'first': [], 'von': [], 'last': [r'{The \TUB{} Team}'], 'jr': []}),

    (r'{\LaTeX} project team',
     {'first': [r'{\LaTeX}'], 'von': ['project'], 'last': ['team'], 'jr': []}),

    (r'{\NTG{} \TeX{} future working group}',
     {'first': [], 'von': [], 'last': [r'{\NTG{} \TeX{} future working group}'], 'jr': []}),

    (r'{{\LaTeX\,3} Project Team}',
     {'first': [], 'von': [], 'last': [r'{{\LaTeX\,3} Project Team}'], 'jr': []}),

    (r'Johansen Kyle, Derik Mamania M.',
     {'first': ['Derik', 'Mamania', 'M.'], 'von': [], 'last': ['Johansen', 'Kyle'], 'jr': []}),

    (r"Johannes Adam Ferdinand Alois Josef Maria Marko d'Aviano Pius von und zu Liechtenstein",
     {'first': ['Johannes', 'Adam', 'Ferdinand', 'Alois', 'Josef', 'Maria', 'Marko'],
      'von': ["d'Aviano", 'Pius', 'von', 'und', 'zu'], 'last': ['Liechtenstein'], 'jr': []}),

    (r"Brand\~{a}o, F",
     {'first': ['F'], 'von': [], 'last': ['Brand\\', '{a}o'], 'jr': []}),
)


if __name__ == '__main__':
    unittest.main()



import unittest
import bibtexparser
from bibtexparser.bibdatabase import BibDatabase
from collections import OrderedDict


class TestPreambleParse(unittest.TestCase):
    def test_single_preamble_parse_count(self):
        bibtex_str = '@preamble{" a "}\n\n'
        bib_database = bibtexparser.loads(bibtex_str)
        self.assertEqual(len(bib_database.preambles), 1)

    def test_multiple_preamble_parse_count(self):
        bibtex_str = '@preamble{" a "}\n\n@preamble{"b"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str)
        self.assertEqual(len(bib_database.preambles), 2)

    def test_single_preamble_parse(self):
        bibtex_str = '@preamble{" a "}\n\n'
        bib_database = bibtexparser.loads(bibtex_str)
        expected = [' a ']
        self.assertEqual(bib_database.preambles, expected)

    def test_multiple_preamble_parse(self):
        bibtex_str = '@preamble{" a "}\n\n@preamble{"b"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str)
        expected = [' a ', 'b']
        self.assertEqual(bib_database.preambles, expected)


class TestPreambleWrite(unittest.TestCase):
    def test_single_preamble_write(self):
        bib_database = BibDatabase()
        bib_database.preambles = [' a ']
        result = bibtexparser.dumps(bib_database)
        expected = '@preamble{" a "}\n\n'
        self.assertEqual(result, expected)

    def test_multiple_string_write(self):
        bib_database = BibDatabase()
        bib_database.preambles = [' a ', 'b']
        result = bibtexparser.dumps(bib_database)
        expected = '@preamble{" a "}\n\n@preamble{"b"}\n\n'
        self.assertEqual(result, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from __future__ import unicode_literals
import unittest

from bibtexparser.bibtexexpression import BibtexExpression


class TestBibtexExpression(unittest.TestCase):

    def setUp(self):
        self.expr = BibtexExpression()

    def test_minimal(self):
        result = self.expr.entry.parse_string('@journal{key, name = 123 }')
        self.assertEqual(result.get('EntryType'), 'journal')
        self.assertEqual(result.get('Key'), 'key')
        self.assertEqual(result.get('Fields'), {'name': '123'})

    def test_capital_type(self):
        result = self.expr.entry.parse_string('@JOURNAL{key, name = 123 }')
        self.assertEqual(result.get('EntryType'), 'JOURNAL')

    def test_capital_key(self):
        result = self.expr.entry.parse_string('@journal{KEY, name = 123 }')
        self.assertEqual(result.get('Key'), 'KEY')

    def test_braced(self):
        result = self.expr.entry.parse_string('@journal{key, name = {abc} }')
        self.assertEqual(result.get('Fields'), {'name': 'abc'})

    def test_braced_with_new_line(self):
        result = self.expr.entry.parse_string(
            '@journal{key, name = {abc\ndef} }')
        self.assertEqual(result.get('Fields'), {'name': 'abc\ndef'})

    def test_braced_unicode(self):
        result = self.expr.entry.parse_string(
            '@journal{key, name = {àbcđéf} }')
        self.assertEqual(result.get('Fields'), {'name': 'àbcđéf'})

    def test_quoted(self):
        result = self.expr.entry.parse_string('@journal{key, name = "abc" }')
        self.assertEqual(result.get('Fields'), {'name': 'abc'})

    def test_quoted_with_new_line(self):
        result = self.expr.entry.parse_string(
            '@journal{key, name = "abc\ndef" }')
        self.assertEqual(result.get('Fields'), {'name': 'abc\ndef'})

    def test_quoted_with_unicode(self):
        result = self.expr.entry.parse_string(
            '@journal{key, name = "àbcđéf" }')
        self.assertEqual(result.get('Fields'), {'name': 'àbcđéf'})

    def test_entry_declaration_after_space(self):
        self.expr.entry.parse_string('  @journal{key, name = {abcd}}')

    def test_entry_declaration_no_key(self):
        with self.assertRaises(self.expr.ParseException):
            self.expr.entry.parse_string('@misc{name = {abcd}}')

    def test_entry_declaration_no_key_new_line(self):
        with self.assertRaises(self.expr.ParseException):
            self.expr.entry.parse_string('@misc{\n name = {abcd}}')

    def test_entry_declaration_no_key_comma(self):
        with self.assertRaises(self.expr.ParseException):
            self.expr.entry.parse_string('@misc{, \nname = {abcd}}')

    def test_entry_declaration_no_key_keyvalue_without_space(self):
        with self.assertRaises(self.expr.ParseException):
            self.expr.entry.parse_string('@misc{\nname=aaa}')

    def test_entry_declaration_key_with_whitespace(self):
        with self.assertRaises(self.expr.ParseException):
            self.expr.entry.parse_string('@misc{ xx yy, \n name = aaa}')

    def test_string_declaration_after_space(self):
        self.expr.string_def.parse_string('  @string{ name = {abcd}}')

    def test_preamble_declaration_after_space(self):
        self.expr.preamble_decl.parse_string('  @preamble{ "blah blah " }')

    def test_declaration_after_space(self):
        keys = []
        self.expr.entry.add_parse_action(
            lambda s, l, t: keys.append(t.get('Key'))
        )
        self.expr.main_expression.parse_string(' @journal{key, name = {abcd}}')
        self.assertEqual(keys, ['key'])

    def test_declaration_after_space_and_comment(self):
        keys = []
        self.expr.entry.add_parse_action(
            lambda s, l, t: keys.append(t.get('Key'))
        )
        self.expr.main_expression.parse_string(
            '% Implicit comment\n @article{key, name={abcd}}'
        )
        self.assertEqual(keys, ['key'])



import unittest
from bibtexparser.bibdatabase import (BibDatabase, BibDataString,
                                      BibDataStringExpression)


class TestBibDatabase(unittest.TestCase):
    entries = [{'ENTRYTYPE': 'book',
                'year': '1987',
                'edition': '2',
                'publisher': 'Wiley Edition',
                'ID': 'Bird1987',
                'volume': '1',
                'title': 'Dynamics of Polymeric Liquid',
                'author': 'Bird, R.B. and Armstrong, R.C. and Hassager, O.'
                }]

    def test_entries_list_method(self):
        bib_db = BibDatabase()
        bib_db.entries = self.entries
        self.assertEqual(bib_db.entries, bib_db.get_entry_list())

    def test_entries_dict_prop(self):
        bib_db = BibDatabase()
        bib_db.entries = self.entries
        self.assertEqual(bib_db.entries_dict, bib_db.get_entry_dict())


class TestBibDataString(unittest.TestCase):

    def setUp(self):
        self.bd = BibDatabase()

    def test_name_is_lower(self):
        bds = BibDataString(self.bd, 'nAmE')
        self.assertTrue(bds.name.islower())

    def test_raises_KeyError(self):
        bds = BibDataString(self.bd, 'name')
        with self.assertRaises(KeyError):
            bds.get_value()

    def test_get_value(self):
        bds = BibDataString(self.bd, 'name')
        self.bd.strings['name'] = 'value'
        self.assertEqual(bds.get_value(), 'value')

    def test_expand_string(self):
        bds = BibDataString(self.bd, 'name')
        self.bd.strings['name'] = 'value'
        self.assertEqual(BibDataString.expand_string('name'), 'name')
        self.assertEqual(BibDataString.expand_string(bds), 'value')

    def test_get_value_string_is_defined_by_expression(self):
        self.bd.strings['name'] = 'string'
        exp = BibDataStringExpression(['this is a ',
                                       BibDataString(self.bd, 'name')])
        self.bd.strings['exp'] = exp
        bds = BibDataString(self.bd, 'exp')
        self.assertEqual(bds.get_value(), 'this is a string')

    def test_strings_are_equal_iif_name_is_equal(self):
        self.bd.strings['a'] = 'foo'
        self.bd.strings['b'] = 'foo'
        a1 = BibDataString(self.bd, 'a')
        a2 = BibDataString(self.bd, 'a')
        b = BibDataString(self.bd, 'b')
        self.assertEqual(a1, a2)
        self.assertNotEqual(a1, b)
        self.assertNotEqual(a1, b)
        self.assertNotEqual(a1, "foo")


class TestBibDataStringExpression(unittest.TestCase):

    def setUp(self):
        self.bd = BibDatabase()
        self.bd.strings['name'] = 'value'
        self.bds = BibDataString(self.bd, 'name')

    def test_get_value(self):
        exp = BibDataStringExpression(
            ["The string has value: ", self.bds, '.'])
        self.assertEqual(exp.get_value(), 'The string has value: value.')

    def test_raises_KeyError(self):
        bds = BibDataString(self.bd, 'unknown')
        exp = BibDataStringExpression([bds, self.bds, 'text'])
        with self.assertRaises(KeyError):
            exp.get_value()

    def test_equations_are_equal_iif_same(self):
        a1 = BibDataString(self.bd, 'a')
        a2 = BibDataString(self.bd, 'a')
        exp = BibDataStringExpression([a1, self.bds, 'text'])
        self.assertEqual(exp, BibDataStringExpression([a2, self.bds, 'text']))
        self.assertNotEqual(exp, BibDataStringExpression(['foo', self.bds, 'text']))
        self.assertNotEqual(exp, 'foovaluetext')


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
import tempfile
import unittest
import bibtexparser
from bibtexparser.bwriter import BibTexWriter, SortingStrategy
from bibtexparser.bibdatabase import BibDatabase


class TestBibTexWriter(unittest.TestCase):
    def test_content_entries_only(self):
        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.contents = ['entries']
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{Toto3000,
 author = {Toto, A and Titi, B},
 title = {A title}
}

@article{Wigner1938,
 author = {Wigner, E.},
 doi = {10.1039/TF9383400029},
 issn = {0014-7672},
 journal = {Trans. Faraday Soc.},
 owner = {fr},
 pages = {29--41},
 publisher = {The Royal Society of Chemistry},
 title = {The transition state method},
 volume = {34},
 year = {1938}
}

@book{Yablon2005,
 author = {Yablon, A.D.},
 publisher = {Springer},
 title = {Optical fiber fusion slicing},
 year = {2005}
}
"""
        self.assertEqual(result, expected)

    def test_content_comment_only(self):
        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.contents = ['comments']
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@comment{}

@comment{A comment}

"""
        self.assertEqual(result, expected)

    def test_indent(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'abc123',
                                 'ENTRYTYPE': 'book',
                                 'author': 'test'}]
        writer = BibTexWriter()
        writer.indent = '  '
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
  author = {test}
}
"""
        self.assertEqual(result, expected)

    def test_align_bool(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'abc123',
                                 'ENTRYTYPE': 'book',
                                 'author': 'test',
                                 'thisisaverylongkey': 'longvalue'}]
        writer = BibTexWriter()
        writer.align_values = True
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 author             = {test},
 thisisaverylongkey = {longvalue}
}
"""
        self.assertEqual(result, expected)

        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'veryveryverylongID',
                                 'ENTRYTYPE': 'book',
                                 'a': 'test',
                                 'bb': 'longvalue'}]
        writer = BibTexWriter()
        writer.align_values = True
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{veryveryverylongID,
 a  = {test},
 bb = {longvalue}
}
"""
        self.assertEqual(result, expected)

        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.contents = ['entries']
        writer.align_values = True
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{Toto3000,
 author    = {Toto, A and Titi, B},
 title     = {A title}
}

@article{Wigner1938,
 author    = {Wigner, E.},
 doi       = {10.1039/TF9383400029},
 issn      = {0014-7672},
 journal   = {Trans. Faraday Soc.},
 owner     = {fr},
 pages     = {29--41},
 publisher = {The Royal Society of Chemistry},
 title     = {The transition state method},
 volume    = {34},
 year      = {1938}
}

@book{Yablon2005,
 author    = {Yablon, A.D.},
 publisher = {Springer},
 title     = {Optical fiber fusion slicing},
 year      = {2005}
}
"""
        self.assertEqual(result, expected)

    def test_align_int(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'abc123',
                                 'ENTRYTYPE': 'book',
                                 'author': 'test',
                                 'thisisaverylongkey': 'longvalue'}]
        # Negative value should have no effect
        writer = BibTexWriter()
        writer.align_values = -20
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 author = {test},
 thisisaverylongkey = {longvalue}
}
"""
        self.assertEqual(result, expected)

        # Value smaller than longest field name should only impact the "short" field names
        writer = BibTexWriter()
        writer.align_values = 10
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 author     = {test},
 thisisaverylongkey = {longvalue}
}
"""
        self.assertEqual(result, expected)


        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.contents = ['entries']
        writer.align_values = 15
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{Toto3000,
 author          = {Toto, A and Titi, B},
 title           = {A title}
}

@article{Wigner1938,
 author          = {Wigner, E.},
 doi             = {10.1039/TF9383400029},
 issn            = {0014-7672},
 journal         = {Trans. Faraday Soc.},
 owner           = {fr},
 pages           = {29--41},
 publisher       = {The Royal Society of Chemistry},
 title           = {The transition state method},
 volume          = {34},
 year            = {1938}
}

@book{Yablon2005,
 author          = {Yablon, A.D.},
 publisher       = {Springer},
 title           = {Optical fiber fusion slicing},
 year            = {2005}
}
"""
        self.assertEqual(result, expected)

    def test_entry_separator(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'abc123',
                                 'ENTRYTYPE': 'book',
                                 'author': 'test'}]
        writer = BibTexWriter()
        writer.entry_separator = ''
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 author = {test}
}
"""
        self.assertEqual(result, expected)

    def test_display_order(self):
        with open('bibtexparser/tests/data/multiple_entries_and_comments.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.contents = ['entries']
        writer.display_order = ['year', 'publisher', 'title']
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{Toto3000,
 title = {A title},
 author = {Toto, A and Titi, B}
}

@article{Wigner1938,
 year = {1938},
 publisher = {The Royal Society of Chemistry},
 title = {The transition state method},
 author = {Wigner, E.},
 doi = {10.1039/TF9383400029},
 issn = {0014-7672},
 journal = {Trans. Faraday Soc.},
 owner = {fr},
 pages = {29--41},
 volume = {34}
}

@book{Yablon2005,
 year = {2005},
 publisher = {Springer},
 title = {Optical fiber fusion slicing},
 author = {Yablon, A.D.}
}
"""
        self.assertEqual(result, expected)

    def test_align_multiline_values(self):
        with open('bibtexparser/tests/data/article_multilines.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.align_multiline_values = True
        writer.display_order = ["author", "title", "year", "journal", "abstract", "comments", "keyword"]
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@article{Cesar2013,
 author = {Jean César},
 title = {A mutline line title is very amazing. It should be
          long enough to test multilines... with two lines or should we
          even test three lines... What an amazing title.},
 year = {2013},
 journal = {Nice Journal},
 abstract = {This is an abstract. This line should be long enough to test
             multilines... and with a french érudit word},
 comments = {A comment},
 keyword = {keyword1, keyword2,
            multiline-keyword1, multiline-keyword2}
}
"""
        self.assertEqual(result, expected)

    def test_align_multiline_values_with_align(self):
        with open('bibtexparser/tests/data/article_multilines.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.align_multiline_values = True
        writer.align_values = True
        writer.display_order = ["author", "title", "year", "journal", "abstract", "comments", "keyword"]
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@article{Cesar2013,
 author   = {Jean César},
 title    = {A mutline line title is very amazing. It should be
             long enough to test multilines... with two lines or should we
             even test three lines... What an amazing title.},
 year     = {2013},
 journal  = {Nice Journal},
 abstract = {This is an abstract. This line should be long enough to test
             multilines... and with a french érudit word},
 comments = {A comment},
 keyword  = {keyword1, keyword2,
             multiline-keyword1, multiline-keyword2}
}
"""
        self.assertEqual(result, expected)

    def test_display_order_sorting(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'abc123',
                                 'ENTRYTYPE': 'book',
                                 'b': 'test2',
                                 'a': 'test1',
                                 'd': 'test4',
                                 'c': 'test3',
                                 'e': 'test5'}]
        # Only 'a' is not ordered. As it's only one element, strategy should not matter.
        for strategy in SortingStrategy:
            writer = BibTexWriter()
            writer.display_order = ['b', 'c', 'd', 'e', 'a']
            writer.display_order_sorting = strategy
            result = bibtexparser.dumps(bib_database, writer)
            expected = \
"""@book{abc123,
 b = {test2},
 c = {test3},
 d = {test4},
 e = {test5},
 a = {test1}
}
"""
            self.assertEqual(result, expected)

        # Test ALPHABETICAL_ASC strategy
        writer = BibTexWriter()
        writer.display_order = ['c']
        writer.display_order_sorting = SortingStrategy.ALPHABETICAL_ASC
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 c = {test3},
 a = {test1},
 b = {test2},
 d = {test4},
 e = {test5}
}
"""
        self.assertEqual(result, expected)

        # Test ALPHABETICAL_DESC strategy
        writer = BibTexWriter()
        writer.display_order = ['c']
        writer.display_order_sorting = SortingStrategy.ALPHABETICAL_DESC
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 c = {test3},
 e = {test5},
 d = {test4},
 b = {test2},
 a = {test1}
}
"""
        self.assertEqual(result, expected)

        # Test PRESERVE strategy
        writer = BibTexWriter()
        writer.display_order = ['c']
        writer.display_order_sorting = SortingStrategy.PRESERVE
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@book{abc123,
 c = {test3},
 b = {test2},
 a = {test1},
 d = {test4},
 e = {test5}
}
"""
        self.assertEqual(result, expected)

    def test_align_multiline_values_with_indent(self):
        with open('bibtexparser/tests/data/article_multilines.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.align_multiline_values = True
        writer.indent = ' ' * 3
        writer.display_order = ["author", "title", "year", "journal", "abstract", "comments", "keyword"]
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@article{Cesar2013,
   author = {Jean César},
   title = {A mutline line title is very amazing. It should be
            long enough to test multilines... with two lines or should we
            even test three lines... What an amazing title.},
   year = {2013},
   journal = {Nice Journal},
   abstract = {This is an abstract. This line should be long enough to test
               multilines... and with a french érudit word},
   comments = {A comment},
   keyword = {keyword1, keyword2,
              multiline-keyword1, multiline-keyword2}
}
"""
        self.assertEqual(result, expected)

    def test_align_multiline_values_with_align_with_indent(self):
        with open('bibtexparser/tests/data/article_multilines.bib') as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)
        writer = BibTexWriter()
        writer.align_multiline_values = True
        writer.indent = ' ' * 3
        writer.align_values = True
        writer.display_order = ["author", "title", "year", "journal", "abstract", "comments", "keyword"]
        result = bibtexparser.dumps(bib_database, writer)
        expected = \
"""@article{Cesar2013,
   author   = {Jean César},
   title    = {A mutline line title is very amazing. It should be
               long enough to test multilines... with two lines or should we
               even test three lines... What an amazing title.},
   year     = {2013},
   journal  = {Nice Journal},
   abstract = {This is an abstract. This line should be long enough to test
               multilines... and with a french érudit word},
   comments = {A comment},
   keyword  = {keyword1, keyword2,
               multiline-keyword1, multiline-keyword2}
}
"""
        self.assertEqual(result, expected)


class TestEntrySorting(unittest.TestCase):
    bib_database = BibDatabase()
    bib_database.entries = [{'ID': 'b',
                             'ENTRYTYPE': 'article'},
                            {'ID': 'c',
                             'ENTRYTYPE': 'book'},
                            {'ID': 'a',
                             'ENTRYTYPE': 'book'}]

    def test_sort_default(self):
        result = bibtexparser.dumps(self.bib_database)
        expected = "@book{a\n}\n\n@article{b\n}\n\n@book{c\n}\n"
        self.assertEqual(result, expected)

    def test_sort_none(self):
        writer = BibTexWriter()
        writer.order_entries_by = None
        result = bibtexparser.dumps(self.bib_database, writer)
        expected = "@article{b\n}\n\n@book{c\n}\n\n@book{a\n}\n"
        self.assertEqual(result, expected)

    def test_sort_id(self):
        writer = BibTexWriter()
        writer.order_entries_by = ('ID', )
        result = bibtexparser.dumps(self.bib_database, writer)
        expected = "@book{a\n}\n\n@article{b\n}\n\n@book{c\n}\n"
        self.assertEqual(result, expected)

    def test_sort_type(self):
        writer = BibTexWriter()
        writer.order_entries_by = ('ENTRYTYPE', )
        result = bibtexparser.dumps(self.bib_database, writer)
        expected = "@article{b\n}\n\n@book{c\n}\n\n@book{a\n}\n"
        self.assertEqual(result, expected)

    def test_sort_type_id(self):
        writer = BibTexWriter()
        writer.order_entries_by = ('ENTRYTYPE', 'ID')
        result = bibtexparser.dumps(self.bib_database, writer)
        expected = "@article{b\n}\n\n@book{a\n}\n\n@book{c\n}\n"
        self.assertEqual(result, expected)

    def test_sort_missing_field(self):
        bib_database = BibDatabase()
        bib_database.entries = [{'ID': 'b',
                                 'ENTRYTYPE': 'article',
                                 'year': '2000'},
                                {'ID': 'c',
                                 'ENTRYTYPE': 'book',
                                 'year': '2010'},
                                {'ID': 'a',
                                 'ENTRYTYPE': 'book'}]
        writer = BibTexWriter()
        writer.order_entries_by = ('year', )
        result = bibtexparser.dumps(bib_database, writer)
        expected = "@book{a\n}\n\n@article{b,\n year = {2000}\n}\n\n@book{c,\n year = {2010}\n}\n"
        self.assertEqual(result, expected)

    def test_unicode_problems(self):
        # See #51
        bibtex = """
        @article{Mesa-Gresa2013,
            abstract = {During a 4-week period half the mice (n = 16) were exposed to EE and the other half (n = 16) remained in a standard environment (SE). Aggr. Behav. 9999:XX-XX, 2013. © 2013 Wiley Periodicals, Inc.},
            author = {Mesa-Gresa, Patricia and P\'{e}rez-Martinez, Asunci\'{o}n and Redolat, Rosa},
            doi = {10.1002/ab.21481},
            file = {:Users/jscholz/Documents/mendeley/Mesa-Gresa, P\'{e}rez-Martinez, Redolat - 2013 - Environmental Enrichment Improves Novel Object Recognition and Enhances Agonistic Behavior.pdf:pdf},
            issn = {1098-2337},
            journal = {Aggressive behavior},
            month = "apr",
            number = {April},
            pages = {269--279},
            pmid = {23588702},
            title = {{Environmental Enrichment Improves Novel Object Recognition and Enhances Agonistic Behavior in Male Mice.}},
            url = {http://www.ncbi.nlm.nih.gov/pubmed/23588702},
            volume = {39},
            year = {2013}
        }
        """
        bibdb = bibtexparser.loads(bibtex)
        with tempfile.TemporaryFile(mode='w+', encoding="utf-8") as bibtex_file:
            bibtexparser.dump(bibdb, bibtex_file)
            # No exception should be raised




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import unittest
import codecs
import warnings

import bibtexparser
from bibtexparser.bparser import BibTexParser
from bibtexparser.bibdatabase import (COMMON_STRINGS, BibDataStringExpression)
from bibtexparser.customization import *
from bibtexparser import customization


def customizations_unicode(record):
    """Use all functions related to specific fields
    + converter to unicode.

    :param record: a record
    :returns: -- customized record
    """

    record = type(record)
    record = author(record)
    record = editor(record)
    record = journal(record)
    record = keyword(record)
    record = link(record)
    record = page_double_hyphen(record)
    record = doi(record)
    record = convert_to_unicode(record)
    return record


def customizations_latex(record):
    """Use all functions related to specific fields
    + converter to latex.

    :param record: a record
    :returns: -- customized record
    """

    record = homogenize_latex_encoding(record)
    record = type(record)
    record = author(record)
    record = editor(record)
    record = journal(record)
    record = keyword(record)
    record = link(record)
    record = page_double_hyphen(record)
    record = doi(record)
    return record


class TestBibtexParserList(unittest.TestCase):

    def test_empty_string(self):
        bib = BibTexParser("", common_strings=False)
        self.assertEqual(bib.entries, [])
        self.assertEqual(bib.comments, [])
        self.assertEqual(bib.preambles, [])
        self.assertEqual(bib.strings, {})

    ###########
    # ARTICLE
    ###########
    # test also that list and dict are equivalent
    def test_article(self):
        with codecs.open('bibtexparser/tests/data/article.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res_list = bib.get_entry_list()
            res_dict = bib.get_entry_dict()
            expected_list = [{'keyword': 'keyword1, keyword2',
                              'ENTRYTYPE': 'article',
                              'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                              'year': '2013',
                              'journal': 'Nice Journal',
                              'ID': 'Cesar2013',
                              'pages': '12-23',
                              'title': 'An amazing title',
                              'comments': 'A comment',
                              'author': 'Jean César',
                              'volume': '12',
                              'month': 'jan'
                              }]
            expected_dict = {'Cesar2013': {'keyword': 'keyword1, keyword2',
                              'ENTRYTYPE': 'article',
                              'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                              'year': '2013',
                              'journal': 'Nice Journal',
                              'ID': 'Cesar2013',
                              'pages': '12-23',
                              'title': 'An amazing title',
                              'comments': 'A comment',
                              'author': 'Jean César',
                              'volume': '12',
                              'month': 'jan'
                              }}
        self.assertEqual(res_list, expected_list)
        self.assertEqual(res_dict, expected_dict)

    def test_article_annotation(self):
        with codecs.open('bibtexparser/tests/data/article_with_annotation.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res_list = bib.get_entry_list()
            res_dict = bib.get_entry_dict()
            expected_list = [{'keyword': 'keyword1, keyword2',
                              'ENTRYTYPE': 'article',
                              'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                              'year': '2013',
                              'journal': 'Nice Journal',
                              'ID': 'Cesar2013',
                              'pages': '12-23',
                              'title': 'An amazing title',
                              'comments': 'A comment',
                              'author': 'Jean César',
                              'author+an': '1=highlight',
                              'volume': '12',
                              'month': 'jan'
                              }]
            expected_dict = {'Cesar2013': {'keyword': 'keyword1, keyword2',
                                           'ENTRYTYPE': 'article',
                                           'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                                           'year': '2013',
                                           'journal': 'Nice Journal',
                                           'ID': 'Cesar2013',
                                           'pages': '12-23',
                                           'title': 'An amazing title',
                                           'comments': 'A comment',
                                           'author': 'Jean César',
                                           'author+an': '1=highlight',
                                           'volume': '12',
                                           'month': 'jan'
                                           }}
        self.assertEqual(res_list, expected_list)
        self.assertEqual(res_dict, expected_dict)

    def test_article_start_bom(self):
        with codecs.open('bibtexparser/tests/data/article_start_with_bom.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                     'ENTRYTYPE': 'article',
                     'pages': '12-23',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean César',
                     'journal': 'Nice Journal',
                     'comments': 'A comment',
                     'month': 'jan',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_article_cust_unicode(self):
        with codecs.open('bibtexparser/tests/data/article.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=customizations_unicode)
            res = bib.get_entry_list()
        expected = [{'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                     'ENTRYTYPE': 'article',
                     'pages': '12--23',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': ['César, Jean'],
                     'journal': {'ID': 'NiceJournal', 'name': 'Nice Journal'},
                     'comments': 'A comment',
                     'month': 'jan',
                     'keyword': ['keyword1', 'keyword2'],
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_article_cust_latex(self):
        with codecs.open('bibtexparser/tests/data/article.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=customizations_latex)
            res = bib.get_entry_list()
        expected = [{'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french {\\\'e}rudit word',
                     'ENTRYTYPE': 'article',
                     'pages': '12--23',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': ['C{\\\'e}sar, Jean'],
                     'journal': {'ID': 'NiceJournal', 'name': 'Nice Journal'},
                     'comments': 'A comment',
                     'month': 'jan',
                     'keyword': ['keyword1', 'keyword2'],
                     'title': '{A}n amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_article_cust_order(self):
        def cust(record):
            record = customization.page_double_hyphen(record)
            record = customization.homogenize_latex_encoding(record)
            record = customization.author(record)
            return record

        def cust2(record):
            record = customization.author(record)
            record = customization.page_double_hyphen(record)
            record = customization.homogenize_latex_encoding(record)
            return record

        with open('bibtexparser/tests/data/multiple_entries.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=cust)
            res = bib.get_entry_list()
        with open('bibtexparser/tests/data/multiple_entries.bib', 'r') as bibfile:
            bib2 = BibTexParser(bibfile.read(), customization=cust2)
            res2 = bib2.get_entry_list()
        self.assertEqual(res, res2)

    def test_article_missing_coma(self):
        with open('bibtexparser/tests/data/article_missing_coma.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean Cesar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Baltazar2013',
                     'year': '2013',
                     'author': 'Jean Baltazar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Aimar2013',
                     'year': '2013',
                     'author': 'Jean Aimar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title',
                     'month': 'january'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Doute2013',
                     'year': '2013',
                     'author': 'Jean Doute',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_oneline(self):
        with open('bibtexparser/tests/data/article_oneline.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean Cesar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Baltazar2013',
                     'year': '2013',
                     'author': 'Jean Baltazar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)


    def test_article_start_with_whitespace(self):
        with open('bibtexparser/tests/data/article_start_with_whitespace.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            self.assertEqual(len(bib.get_entry_list()), 2)

    def test_article_comma_first(self):
        with open('bibtexparser/tests/data/article_comma_first.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'author': 'Jean Cesar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     },
                    {'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'ID': 'Baltazar2013',
                     'year': '2013',
                     'author': 'Jean Baltazar',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title'
                     }]
        self.assertEqual(res, expected)

    def test_article_no_braces(self):
        with open('bibtexparser/tests/data/article_no_braces.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'pages': '12-23',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'month': 'jan',
                     'author': 'Jean C{\\\'e}sar{\\\"u}',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title',
                     'abstract': "This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word",
                     },
                     ]
        self.assertEqual(res, expected)

    def test_article_special_characters(self):
        with open('bibtexparser/tests/data/article_with_special_characters.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': 'Nice Journal',
                     'volume': '12',
                     'pages': '12-23',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'month': 'jan',
                     'author': 'Jean C{\\\'e}sar{\\\"u}',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': 'An amazing title',
                     'abstract': "This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word",
                     },
                     ]
        self.assertEqual(res, expected)

    def test_article_protection_braces(self):
        with open('bibtexparser/tests/data/article_with_protection_braces.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'ENTRYTYPE': 'article',
                     'journal': '{Nice Journal}',
                     'volume': '12',
                     'pages': '12-23',
                     'ID': 'Cesar2013',
                     'year': '2013',
                     'month': 'jan',
                     'author': 'Jean César',
                     'comments': 'A comment',
                     'keyword': 'keyword1, keyword2',
                     'title': '{An amazing title}',
                     'abstract': "This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word",
                     },
                     ]
        self.assertEqual(res, expected)


    ###########
    # BOOK
    ###########
    def test_book(self):
        with open('bibtexparser/tests/data/book.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'ENTRYTYPE': 'book',
                         'year': '1987',
                         'edition': '2',
                         'publisher': 'Wiley Edition',
                         'ID': 'Bird1987',
                         'volume': '1',
                         'title': 'Dynamics of Polymeric Liquid',
                         'author': 'Bird, R.B. and Armstrong, R.C. and Hassager, O.'
                         }]

        self.assertEqual(res, expected)

    def test_book_cust_unicode(self):
        with open('bibtexparser/tests/data/book_capital_AND.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=customizations_unicode)
            res = bib.get_entry_list()
            expected = [{'ENTRYTYPE': 'book',
                         'year': '1987',
                         'edition': '2',
                         'publisher': 'Wiley Edition',
                         'ID': 'Bird1987',
                         'volume': '1',
                         'title': 'Dynamics of Polymeric Liquid',
                         'author': ['Bird, R.B.', 'Armstrong, R.C.', 'Hassager, O.']
                         }]

        self.assertEqual(res, expected)

    def test_book_cust_latex(self):
        with open('bibtexparser/tests/data/book.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=customizations_latex)
            res = bib.get_entry_list()
            expected = [{'ENTRYTYPE': 'book',
                         'year': '1987',
                         'edition': '2',
                         'publisher': 'Wiley Edition',
                         'ID': 'Bird1987',
                         'volume': '1',
                         'title': '{D}ynamics of {P}olymeric {L}iquid',
                         'author': ['Bird, R.B.', 'Armstrong, R.C.', 'Hassager, O.']
                         }]

        self.assertEqual(res, expected)

    def test_traps(self):
        with codecs.open('bibtexparser/tests/data/traps.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'keywords': 'keyword1, keyword2',
                         'ENTRYTYPE': 'article',
                         'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                         'year': '2013',
                         'journal': 'Nice Journal',
                         'ID': 'Laide2013',
                         'pages': '12-23',
                         'title': '{An} amazing {title}',
                         'comments': 'A comment',
                         'author': 'Jean Laid{\\\'e},\nBen Loaeb',
                         'volume': 'n.s.~2',
                         'month': 'jan'
                         }]
        self.assertEqual(res, expected)

    def test_features(self):
        with open('bibtexparser/tests/data/features.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'ENTRYTYPE': 'inproceedings',
                         'year': '2014',
                         'title': 'Cool Stuff',
                         'author': 'John',
                         'ID': 'mykey',
                         'booktitle': 'My International Conference',
                         }]
        self.assertEqual(res, expected)

    def test_features2(self):
        with open('bibtexparser/tests/data/features2.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'ENTRYTYPE': 'inproceedings',
                         'year': '2014',
                         'title': 'Cool Stuff',
                         'author': 'John Doe',
                         'ID': 'mykey',
                         'booktitle': 'My International Conference',
                         'note': 'Email: John.Doe@example.com',
                         'pages': '1--10',
                         }]
        self.assertEqual(res, expected)

    def test_nonstandard_ignored(self):
        with open('bibtexparser/tests/data/wrong.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'author': 'correct',
                         'ID': 'bar',
                         'ENTRYTYPE': 'article'}]
        self.assertEqual(res, expected)

    def test_nonstandard_not_ignored(self):
        with open('bibtexparser/tests/data/wrong.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), ignore_nonstandard_types=False)
            res = bib.get_entry_list()
        self.assertEqual(len(res), 2)

    def test_encoding(self):
        with codecs.open('bibtexparser/tests/data/encoding.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
            expected = [{'keywords': 'keyword1, keyword2',
                         'ENTRYTYPE': 'article',
                         'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
                         'year': '2013',
                         'journal': 'Elémentaire',
                         'ID': 'Cesar_2013',
                         'pages': '12-23',
                         'title': 'An amazing title: à',
                         'comments': 'A comment',
                         'author': 'Jean César',
                         'volume': '12',
                         'month': 'jan'
                         }]
        self.assertEqual(res, expected)

    def test_encoding_with_homogenize(self):
        with codecs.open('bibtexparser/tests/data/encoding.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=homogenize_latex_encoding)
            res = bib.get_entry_list()
            expected = [{'keywords': 'keyword1, keyword2',
                         'ENTRYTYPE': 'article',
                         'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french {\\\'e}rudit word',
                         'year': '2013',
                         'journal': 'El{\\\'e}mentaire',
                         'ID': 'Cesar_2013',
                         'pages': '12-23',
                         'title': '{A}n amazing title: {\\`a}',
                         'comments': 'A comment',
                         'author': 'Jean C{\\\'e}sar',
                         'volume': '12',
                         'month': 'jan'
                         }]
        self.assertEqual(res, expected)

    def test_field_name_with_dash_underscore(self):
        with open('bibtexparser/tests/data/article_field_name_with_underscore.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read())
        res = bib.get_entry_list()
        expected = [{
            'keyword': 'keyword1, keyword2',
            'ENTRYTYPE': 'article',
            'year': '2013',
            'journal': 'Nice Journal',
            'ID': 'Cesar2013',
            'pages': '12-23',
            'title': 'An amazing title',
            'comments': 'A comment',
            'author': 'Jean César',
            'volume': '12',
            'strange_field_name': 'val',
            'strange-field-name2': 'val2',
            }]
        self.assertEqual(res, expected)

    def test_string_definitions(self):
        with open('bibtexparser/tests/data/article_with_strings.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read(), common_strings=True)
        res = dict(bib.strings)
        expected = COMMON_STRINGS.copy()
        expected.update({
                'nice_journal': 'Nice Journal',
                'jean': 'Jean',
                'cesar': "César",
                })
        self.assertEqual(res, expected)

    def test_string_is_interpolated(self):
        with open('bibtexparser/tests/data/article_with_strings.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read(), common_strings=True,
                               interpolate_strings=True)
        res = bib.get_entry_list()
        expected = [{
            'keyword': 'keyword1, keyword2',
            'ENTRYTYPE': 'article',
            'year': '2013',
            'month': 'January',
            'journal': 'Nice Journal',
            'ID': 'Cesar2013',
            'pages': '12-23',
            'title': 'An amazing title',
            'comments': 'A comment',
            'author': 'Jean César',
            'volume': '12',
            }]
        self.assertEqual(res, expected)

    def test_string_is_not_interpolated(self):
        with open('bibtexparser/tests/data/article_with_strings.bib', 'r',
                  encoding="utf-8") as bibfile:
            bib = BibTexParser(bibfile.read(), common_strings=True,
                               interpolate_strings=False)
        res = bib.get_entry_list()[0]
        self.assertIsInstance(res['month'], BibDataStringExpression)
        self.assertEqual(len(res['month'].expr), 1)
        self.assertEqual(res['month'].get_value(), 'January')
        self.assertIsInstance(res['author'], BibDataStringExpression)
        self.assertEqual(len(res['author'].expr), 3)
        self.assertEqual(res['author'].get_value(), 'Jean César')
        self.assertIsInstance(res['journal'], BibDataStringExpression)
        self.assertEqual(len(res['journal'].expr), 1)
        self.assertEqual(res['journal'].get_value(), 'Nice Journal')

    def test_comments_spaces_and_declarations(self):
        with codecs.open(
                'bibtexparser/tests/data/comments_spaces_and_declarations.bib',
                'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
        res_dict = bib.get_entry_dict()
        expected_dict = {'Cesar2013': {
            'keyword': 'keyword1, keyword2',
            'ENTRYTYPE': 'article',
            'abstract': 'This is an abstract. This line should be long enough to test\nmultilines... and with a french érudit word',
            'year': '2013',
            'journal': 'Nice Journal',
            'ID': 'Cesar2013',
            'pages': '12-23',
            'title': 'A great title',
            'comments': 'A comment',
            'author': 'Jean César',
            'volume': '12',
            'month': 'jan'
        }}
        self.assertEqual(res_dict, expected_dict)
        self.assertEqual(bib.preambles, ["Blah blah"])

    def test_does_not_fail_on_non_bibtex_with_partial(self):
        bibraw = '''@misc{this looks,
          like = a = bibtex file but
              , is not a real one!
        '''
        parser = BibTexParser(common_strings=False)
        bib = parser.parse(bibraw, partial=False)
        self.assertEqual(bib.entries, [])
        self.assertEqual(bib.preambles, [])
        self.assertEqual(bib.strings, {})
        self.assertEqual(bib.comments, [
            '@misc{this looks,\n'
            '          like = a = bibtex file but\n'
            '              , is not a real one!'])

    def test_no_citekey_parsed_as_comment(self):
        bib = BibTexParser('@BOOK{, title = "bla"}', common_strings=False)
        self.assertEqual(bib.entries, [])
        self.assertEqual(bib.preambles, [])
        self.assertEqual(bib.strings, {})
        self.assertEqual(bib.comments, ['@BOOK{, title = "bla"}'])

    def test_parsing_just_once_not_raising_warnings_with_default_settings(self):
        parser = BibTexParser()

        with open('bibtexparser/tests/data/article_comma_normal_single.bib',
                  'r', encoding='utf-8') as bibfile:
            with warnings.catch_warnings(record=True) as warning:
                warnings.simplefilter("always")
                bibtexparser.load(bibfile, parser)
                assert len(warning) == 0

    def test_parsing_twice_raise_warnings_with_default_settings(self):
        parser = BibTexParser()

        with open('bibtexparser/tests/data/article_comma_normal_single.bib',
                  'r', encoding='utf-8') as bibfile_first, \
            open('bibtexparser/tests/data/article_comma_normal_multiple.bib', 'r', encoding='utf-8') \
                as bibfile_second:
            with warnings.catch_warnings(record=True) as warning:
                warnings.simplefilter("always")
                bibtexparser.load(bibfile_first, parser)
                bibtexparser.load(bibfile_second, parser)
                assert len(warning) != 0

    def test_parsing_three_times_raise_warnings_only_once_with_default_settings(self):
        parser = BibTexParser()

        with open('bibtexparser/tests/data/article_comma_normal_single.bib',
                  'r', encoding='utf-8') as bibfile_first, \
            open('bibtexparser/tests/data/article_comma_normal_multiple.bib', 'r', encoding='utf-8') \
                as bibfile_second, \
                open('bibtexparser/tests/data/article.bib', 'r', encoding='utf-8') as bibfile_third:
            with warnings.catch_warnings(record=True) as warning:
                warnings.simplefilter("always")
                bibtexparser.load(bibfile_first, parser)
                bibtexparser.load(bibfile_second, parser)
                bibtexparser.load(bibfile_third, parser)
                assert len(warning) == 1

    def test_parsing_three_times_not_raising_a_warning_if_expect_multiple_parse_is_true(self):
        parser = BibTexParser()
        parser.expect_multiple_parse = True

        with open('bibtexparser/tests/data/article_comma_normal_single.bib',
                  'r', encoding='utf-8') as bibfile_first, \
            open('bibtexparser/tests/data/article_comma_normal_multiple.bib', 'r', encoding='utf-8') \
                as bibfile_second, \
                open('bibtexparser/tests/data/article.bib', 'r', encoding='utf-8') as bibfile_third:
            with warnings.catch_warnings(record=True) as warning:
                warnings.simplefilter("always")
                bibtexparser.load(bibfile_first, parser)
                bibtexparser.load(bibfile_second, parser)
                bibtexparser.load(bibfile_third, parser)
                assert len(warning) == 0


if __name__ == '__main__':
    unittest.main()



import unittest
import bibtexparser
from bibtexparser.bparser import BibTexParser
from tempfile import TemporaryFile


class TestBibtexParserParserMethods(unittest.TestCase):
    input_file_path = 'bibtexparser/tests/data/book.bib'
    input_bom_file_path = 'bibtexparser/tests/data/book_bom.bib'
    entries_expected = [{'ENTRYTYPE': 'book',
                         'year': '1987',
                         'edition': '2',
                         'publisher': 'Wiley Edition',
                         'ID': 'Bird1987',
                         'volume': '1',
                         'title': 'Dynamics of Polymeric Liquid',
                         'author': 'Bird, R.B. and Armstrong, R.C. and Hassager, O.',
                        }]

    def test_parse_immediately(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_str = bibtex_file.read()
        bibtex_database = BibTexParser(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_str(self):
        parser = BibTexParser()
        with open(self.input_file_path) as bibtex_file:
            bibtex_str = bibtex_file.read()
        bibtex_database = parser.parse(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_bom_str(self):
        parser = BibTexParser()
        with open(self.input_bom_file_path, encoding="utf-8") as bibtex_file:
            bibtex_str = bibtex_file.read()
            bibtex_database = parser.parse(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_empty(self):
        parser = BibTexParser()
        bibtex_database_from_str = parser.parse('')
        self.assertEqual(bibtex_database_from_str.entries, [])

        with open('bibtexparser/tests/data/empty.bib') as bibtex_file:
            bibtex_database_from_file = parser.parse_file(bibtex_file)
        self.assertEqual(bibtex_database_from_file.entries, [])

    def test_parse_bom_bytes(self):
        parser = BibTexParser()
        with open(self.input_bom_file_path, 'rb') as bibtex_file:
            bibtex_str = bibtex_file.read()
            bibtex_database = parser.parse(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_file(self):
        parser = BibTexParser()
        with open(self.input_file_path) as bibtex_file:
            bibtex_database = parser.parse_file(bibtex_file)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_str_module(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_str = bibtex_file.read()
        bibtex_database = bibtexparser.loads(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

    def test_parse_file_module(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_database = bibtexparser.load(bibtex_file)
        self.assertEqual(bibtex_database.entries, self.entries_expected)


class TestBibtexparserWriteMethods(unittest.TestCase):
    input_file_path = 'bibtexparser/tests/data/book.bib'
    expected = \
"""@book{Bird1987,
 author = {Bird, R.B. and Armstrong, R.C. and Hassager, O.},
 edition = {2},
 publisher = {Wiley Edition},
 title = {Dynamics of Polymeric Liquid},
 volume = {1},
 year = {1987}
}
"""

    def test_write_str(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_database = bibtexparser.load(bibtex_file)
        result = bibtexparser.dumps(bibtex_database)
        self.assertEqual(result, self.expected)

    def test_write_file(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_database = bibtexparser.load(bibtex_file)

        with TemporaryFile(mode='w+') as bibtex_out_file:
            bibtexparser.dump(bibtex_database, bibtex_out_file)
            bibtex_out_file.seek(0)
            bibtex_out_str = bibtex_out_file.read()

        self.assertEqual(bibtex_out_str, self.expected)

class TestBibtexparserFieldNames(unittest.TestCase):
    input_file_path = 'bibtexparser/tests/data/fieldname.bib'
    entries_expected = [{'ENTRYTYPE': 'book',
                         'ID': 'Bird1987',
                         'dc.date': '2004-01'
                        }]

    def test_parse_immediately(self):
        with open(self.input_file_path) as bibtex_file:
            bibtex_str = bibtex_file.read()
        bibtex_database = BibTexParser(bibtex_str)
        self.assertEqual(bibtex_database.entries, self.entries_expected)

if __name__ == '__main__':
    unittest.main()



import io
import unittest
import codecs
import bibtexparser
from bibtexparser.bibdatabase import (BibDatabase, BibDataString,
                                      BibDataStringExpression)
from bibtexparser.bparser import BibTexParser
from bibtexparser.bwriter import BibTexWriter
from collections import OrderedDict


class TestStringParse(unittest.TestCase):

    def test_single_string_parse_count(self):
        parser = BibTexParser(common_strings=False)
        bibtex_str = '@string{name1 = "value1"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str, parser)
        self.assertEqual(len(bib_database.strings), 1)

    def test_multiple_string_parse_count(self):
        parser = BibTexParser(common_strings=False)
        bibtex_str = '@string{name1 = "value1"}\n\n@string{name2 = "value2"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str, parser)
        self.assertEqual(len(bib_database.strings), 2)

    def test_single_string_parse(self):
        parser = BibTexParser(common_strings=False)
        bibtex_str = '@string{name1 = "value1"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str, parser)
        expected = {'name1': 'value1'}
        self.assertEqual(bib_database.strings, expected)

    def test_multiple_string_parse(self):
        parser = BibTexParser(common_strings=False)
        bibtex_str = '@string{name1 = "value1"}\n\n@string{name2 = "value2"}\n\n'
        bib_database = bibtexparser.loads(bibtex_str, parser)
        expected = OrderedDict()
        expected['name1'] = 'value1'
        expected['name2'] = 'value2'
        self.assertEqual(bib_database.strings, expected)

    def test_string_braces(self):
        with codecs.open('bibtexparser/tests/data/string.bib', 'r', 'utf-8') as bibfile:
            bib = BibTexParser(bibfile.read())
            res = bib.get_entry_list()
        expected = [{'author': 'Sang Kil Cha and Maverick Woo and David Brumley',
		     'ID': 'cha:oakland15',
		     'year': '2015',
		     'booktitle': 'Proceedings of the {IEEE} Symposium on Security and Privacy',
		     'title': '{Program-Adaptive Mutational Fuzzing}',
	             'ENTRYTYPE': 'inproceedings',
		     'pages': '725--741'
                     }]
        self.assertEqual(res, expected)

    def test_string_parse_accept_chars(self):
        parser = BibTexParser(common_strings=False)
        bibtex_str = '@string{pub-ieee-std = {IEEE}}\n\n@string{pub-ieee-std:adr = {New York, NY, USA}}'
        bib_database = bibtexparser.loads(bibtex_str, parser)
        self.assertEqual(len(bib_database.strings), 2)
        expected = OrderedDict()
        expected['pub-ieee-std'] = 'IEEE'
        expected['pub-ieee-std:adr'] = 'New York, NY, USA'
        self.assertEqual(bib_database.strings, expected)


class TestStringWrite(unittest.TestCase):

    def test_single_string_write(self):
        bib_database = BibDatabase()
        bib_database.strings['name1'] = 'value1'
        result = bibtexparser.dumps(bib_database)
        expected = '@string{name1 = {value1}}\n\n'
        self.assertEqual(result, expected)

    def test_multiple_string_write(self):
        bib_database = BibDatabase()
        bib_database.strings['name1'] = 'value1'
        bib_database.strings['name2'] = 'value2'  # Order is important!
        result = bibtexparser.dumps(bib_database)
        expected = '@string{name1 = {value1}}\n\n@string{name2 = {value2}}\n\n'
        self.assertEqual(result, expected)

    def test_ignore_common_strings(self):
        bib_database = BibDatabase()
        bib_database.load_common_strings()
        result = bibtexparser.dumps(bib_database)
        self.assertEqual(result, '')

    def test_ignore_common_strings_only_if_not_overloaded(self):
        bib_database = BibDatabase()
        bib_database.load_common_strings()
        bib_database.strings['jan'] = 'Janvier'
        result = bibtexparser.dumps(bib_database)
        self.assertEqual(result, '@string{jan = {Janvier}}\n\n')

    def test_write_common_strings(self):
        bib_database = BibDatabase()
        bib_database.load_common_strings()
        writer = BibTexWriter(write_common_strings=True)
        result = bibtexparser.dumps(bib_database, writer=writer)
        with io.open('bibtexparser/tests/data/common_strings.bib') as f:
            expected = f.read()
        self.assertEqual(result, expected)

    def test_write_dependent_strings(self):
        bib_database = BibDatabase()
        bib_database.strings['title'] = 'Mr'
        expr = BibDataStringExpression([BibDataString(bib_database, 'title'), 'Smith'])
        bib_database.strings['name'] = expr
        result = bibtexparser.dumps(bib_database)
        expected = '@string{title = {Mr}}\n\n@string{name = title # {Smith}}\n\n'
        self.assertEqual(result, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Author: Francois Boulogne
# License:

from __future__ import unicode_literals

import unittest
import os
import io

from bibtexparser.bparser import BibTexParser
from bibtexparser.bwriter import BibTexWriter, to_bibtex
from bibtexparser.customization import author


def _data_path(filename):
    return os.path.join('bibtexparser/tests/data', filename)


class TestBibtexWriterList(unittest.TestCase):

    def test_article(self):
        with io.open(_data_path('article.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('article_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_article_with_annotation(self):
        with io.open(_data_path('article_with_annotation.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('article_with_annotation_output.bib'), 'r') \
                as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_book(self):
        with io.open(_data_path('book.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('book_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_comma_first(self):
        with io.open(_data_path('book.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('book_comma_first.bib'), 'r') as bibfile:
            expected = bibfile.read()
        writer = BibTexWriter()
        writer.indent = '   '
        writer.comma_first = True
        result = writer.write(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_multiple(self):
        with io.open(_data_path('multiple_entries.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('multiple_entries_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_exception_typeerror(self):
        with io.open(_data_path('article.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), customization=author)
        self.assertRaises(TypeError, to_bibtex, bib)

    def test_with_strings(self):
        with io.open(_data_path('article_with_strings.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), common_strings=True,
                               interpolate_strings=False)
        with io.open(_data_path(
                'article_with_strings_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        result = to_bibtex(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_trailing_comma(self):
        with io.open(_data_path('article.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('article_trailing_comma_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        writer = BibTexWriter()
        writer.add_trailing_comma = True
        result = writer.write(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)

    def test_comma_first_and_trailing_comma(self):
        with io.open(_data_path('article.bib'), 'r') as bibfile:
            bib = BibTexParser(bibfile.read())

        with io.open(_data_path('article_comma_first_and_trailing_comma_output.bib'), 'r') as bibfile:
            expected = bibfile.read()
        writer = BibTexWriter()
        writer.add_trailing_comma = True
        writer.comma_first = True
        result = writer.write(bib)
        self.maxDiff = None
        self.assertEqual(expected, result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>
#
# Author: Francois Boulogne <fboulogne at sciunto dot org>, 2012

from __future__ import unicode_literals
import unittest

from bibtexparser.latexenc import (string_to_latex, latex_to_unicode,
                                   protect_uppercase)


class TestLatexConverter(unittest.TestCase):

    def test_accent(self):
        string = 'à é è ö'
        result = string_to_latex(string)
        expected = "{\`a} {\\\'e} {\`e} {\\\"o}"
        self.assertEqual(result, expected)

    def test_special_caracter(self):
        string = 'ç'
        result = string_to_latex(string)
        expected = '{\c c}'
        self.assertEqual(result, expected)


class TestUppercaseProtection(unittest.TestCase):

    def test_uppercase(self):
        string = 'An upPer Case A'
        result = protect_uppercase(string)
        expected = '{A}n up{P}er {C}ase {A}'
        self.assertEqual(result, expected)

    def test_lowercase(self):
        string = 'a'
        result = protect_uppercase(string)
        expected = 'a'
        self.assertEqual(result, expected)

    def test_alreadyprotected(self):
        string = '{A}, m{A}gnificient, it is a {A}...'
        result = protect_uppercase(string)
        expected = '{A}, m{A}gnificient, it is a {A}...'
        self.assertEqual(result, expected)

    def test_traps(self):
        string = '{A, m{Agnificient, it is a {A'
        result = protect_uppercase(string)
        expected = '{A, m{Agnificient, it is a {A'
        self.assertEqual(result, expected)

    def test_traps2(self):
        string = 'A}, mA}gnificient, it is a A}'
        result = protect_uppercase(string)
        expected = 'A}, mA}gnificient, it is a A}'
        self.assertEqual(result, expected)


class TestUnicodeConversion(unittest.TestCase):

    def test_accents(self):
        string = "{\`a} {\\\'e} {\`e} {\\\"o}"
        result = latex_to_unicode(string)
        expected = 'à é è ö'
        self.assertEqual(result, expected)

    def test_ignores_trailing_modifier(self):
        string = "a\\\'"
        result = latex_to_unicode(string)
        expected = 'a'
        self.assertEqual(result, expected)

    def test_special_caracter(self):
        string = '{\c c}'
        result = latex_to_unicode(string)
        expected = 'ç'
        self.assertEqual(result, expected)

    def test_does_not_modify_existing_combining(self):
        string = b'ph\xc6\xa1\xcc\x89'.decode('utf8')
        result = latex_to_unicode(string)
        expected = 'phở'  # normalized
        self.assertEqual(result, expected)

    def test_does_not_modify_two_existing_combining(self):
        string = b'pho\xcc\x9b\xcc\x89'.decode('utf8')
        result = latex_to_unicode(string)
        expected = 'phở'  # normalized
        self.assertEqual(result, expected)


if __name__ == '__main__':
    unittest.main()



import logging
import logging.config
import io

logger = logging.getLogger(__name__)

logging.config.dictConfig({
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': '%(asctime)s [%(levelname)s] %(name)s %(funcName)s:%(lineno)d: %(message)s'
        },
    },
    'handlers': {
        'default': {
            'level':'DEBUG',
            'formatter': 'standard',
            'class':'logging.StreamHandler',
        },
    },
    'loggers': {
        '': {
            'handlers': ['default'],
            'level': 'DEBUG',
            'formatter': 'standard',
            'propagate': True
        }
    }
})


if __name__ == '__main__':
    bibtex_source = """@ARTICLE{Cesar2013,
      author = {Jean César},
      title = {An amazing title},
      year = {2013},
      month = {1},
      volume = {12},
      pages = {12--23},
      journal = {Nice Journal},
      abstract = {This is an abstract. This line should be long enough to test
    	 multilines...},
      comments = {A comment},
      keywords = {keyword1, keyword2},
    }
    """

    from bibtexparser.bparser import BibTexParser

    with io.StringIO(bibtex_source) as f:
        bp = BibTexParser(f.read())
        print(bp.get_entry_list())



import unittest
from bibtexparser.bibdatabase import BibDatabase
from bibtexparser.bparser import BibTexParser


class TestCrossRef(unittest.TestCase):
    def test_crossref(self):
        self.maxDiff = None
        input_file_path = 'bibtexparser/tests/data/crossref_entries.bib'
        entries_expected = {'cr1': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr1',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'year'],
                                    'archiveprefix': 'SomEPrFiX',
                                    'author': 'Graham Gullam',
                                    'crossref': 'cr_m',
                                    'editor': 'Edgar Erbriss',
                                    'origdate': '1955',
                                    'primaryclass': 'SOMECLASS',
                                    'publisher': 'Grimble',
                                    'title': 'Great and Good Graphs',
                                    'year': '1974'},
                            'cr2': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr2',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'year'],
                                    'author': 'Frederick Fumble',
                                    'crossref': 'cr_m',
                                    'editor': 'Edgar Erbriss',
                                    'institution': 'Institution',
                                    'origdate': '1943',
                                    'publisher': 'Grimble',
                                    'school': 'School',
                                    'title': 'Fabulous Fourier Forms',
                                    'year': '1974'},
                            'cr3': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr3',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'year'],
                                    'archiveprefix': 'SomEPrFiX',
                                    'author': 'Arthur Aptitude',
                                    'crossref': 'crt',
                                    'editor': 'Mark Monkley',
                                    'eprinttype': 'sometype',
                                    'origdate': '1934',
                                    'publisher': 'Rancour',
                                    'title': 'Arrangements of All Articles',
                                    'year': '1996'},
                            'cr4': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr4',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'year'],
                                    'author': 'Morris Mumble',
                                    'crossref': 'crn',
                                    'editor': 'Jeremy Jermain',
                                    'origdate': '1911',
                                    'publisher': 'Pillsbury',
                                    'title': 'Enterprising Entities',
                                    'year': '1945'},
                            'cr5': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr5',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'year'],
                                    'author': 'Oliver Ordinary',
                                    'crossref': 'crn',
                                    'editor': 'Jeremy Jermain',
                                    'origdate': '1919',
                                    'publisher': 'Pillsbury',
                                    'title': 'Questionable Quidities',
                                    'year': '1945'},
                            'cr6': {'ENTRYTYPE': 'inproceedings',
                                    'ID': 'cr6',
                                    '_FROM_CROSSREF': ['address',
                                                       'editor',
                                                       'eventdate',
                                                       'eventtitle',
                                                       'publisher',
                                                       'venue'],
                                    'address': 'Address',
                                    'author': 'Author, Firstname',
                                    'booktitle': 'Manual booktitle',
                                    'crossref': 'cr6i',
                                    'editor': 'Editor',
                                    'eventdate': '2009-08-21/2009-08-24',
                                    'eventtitle': 'Title of the event',
                                    'pages': '123--',
                                    'publisher': 'Publisher of proceeding',
                                    'title': 'Title of inproceeding',
                                    'venue': 'Location of event',
                                    'year': '2009'},
                            'cr6i': {'ENTRYTYPE': 'proceedings',
                                     'ID': 'cr6i',
                                     'address': 'Address',
                                     'author': 'Spurious Author',
                                     'editor': 'Editor',
                                     'eventdate': '2009-08-21/2009-08-24',
                                     'eventtitle': 'Title of the event',
                                     'publisher': 'Publisher of proceeding',
                                     'title': 'Title of proceeding',
                                     'venue': 'Location of event',
                                     'year': '2009'},
                            'cr7': {'ENTRYTYPE': 'inbook',
                                    'ID': 'cr7',
                                    '_FROM_CROSSREF': ['publisher', 'subtitle', 'titleaddon', 'verba'],
                                    'author': 'Author, Firstname',
                                    'crossref': 'cr7i',
                                    'pages': '123--126',
                                    'publisher': 'Publisher of proceeding',
                                    'subtitle': 'Book Subtitle',
                                    'title': 'Title of Book bit',
                                    'titleaddon': 'Book Titleaddon',
                                    'verba': 'String',
                                    'year': '2010'},
                            'cr7i': {'ENTRYTYPE': 'book',
                                     'ID': 'cr7i',
                                     'author': 'Brian Bookauthor',
                                     'publisher': 'Publisher of proceeding',
                                     'subtitle': 'Book Subtitle',
                                     'title': 'Book Title',
                                     'titleaddon': 'Book Titleaddon',
                                     'verba': 'String',
                                     'year': '2009'},
                            'cr8': {'ENTRYTYPE': 'incollection',
                                    'ID': 'cr8',
                                    '_FROM_CROSSREF': ['editor', 'publisher', 'subtitle', 'titleaddon'],
                                    'author': 'Smith, Firstname',
                                    'crossref': 'cr8i',
                                    'editor': 'Brian Editor',
                                    'pages': '1--12',
                                    'publisher': 'Publisher of Collection',
                                    'subtitle': 'Book Subtitle',
                                    'title': 'Title of Collection bit',
                                    'titleaddon': 'Book Titleaddon',
                                    'year': '2010'},
                            'cr8i': {'ENTRYTYPE': 'collection',
                                     'ID': 'cr8i',
                                     'editor': 'Brian Editor',
                                     'publisher': 'Publisher of Collection',
                                     'subtitle': 'Book Subtitle',
                                     'title': 'Book Title',
                                     'titleaddon': 'Book Titleaddon',
                                     'year': '2009'},
                            'cr_m': {'ENTRYTYPE': 'book',
                                     'ID': 'cr_m',
                                     'editor': 'Edgar Erbriss',
                                     'publisher': 'Grimble',
                                     'title': 'Graphs of the Continent',
                                     'year': '1974'},
                            'crn': {'ENTRYTYPE': 'book',
                                    'ID': 'crn',
                                    'editor': 'Jeremy Jermain',
                                    'publisher': 'Pillsbury',
                                    'title': 'Vanquished, Victor, Vandal',
                                    'year': '1945'},
                            'crt': {'ENTRYTYPE': 'book',
                                    'ID': 'crt',
                                    'editor': 'Mark Monkley',
                                    'publisher': 'Rancour',
                                    'title': 'Beasts of the Burbling Burns',
                                    'year': '1996'}}
        parser = BibTexParser(add_missing_from_crossref=True, ignore_nonstandard_types=False)
        with open(input_file_path) as bibtex_file:
            bibtex_database = parser.parse_file(bibtex_file)
        self.assertDictEqual(bibtex_database.entries_dict, entries_expected)

    def test_crossref_cascading(self):
        input_file_path = 'bibtexparser/tests/data/crossref_cascading.bib'
        entries_expected = {'r1': {'ENTRYTYPE': 'book',
                                   'ID': 'r1',
                                   '_FROM_CROSSREF': [],
                                   'crossref': 'r2',
                                   'date': '1911'},
                            'r2': {'ENTRYTYPE': 'book',
                                   'ID': 'r2',
                                   '_FROM_CROSSREF': [],
                                   'crossref': 'r3',
                                   'date': '1911'},
                            'r3': {'ENTRYTYPE': 'book',
                                   'ID': 'r3',
                                   '_FROM_CROSSREF': [],
                                   'crossref': 'r4',
                                   'date': '1911'},
                            'r4': {'ENTRYTYPE': 'book',
                                   'ID': 'r4',
                                   'date': '1911'}}

        parser = BibTexParser(add_missing_from_crossref=True)
        with open(input_file_path) as bibtex_file:
            bibtex_database = parser.parse_file(bibtex_file)
        self.assertDictEqual(bibtex_database.entries_dict, entries_expected)

    def test_crossref_cascading_cycle(self):
        input_file_path = 'bibtexparser/tests/data/crossref_cascading_cycle.bib'
        entries_expected = {'circ1': {'ENTRYTYPE': 'book',
                                      'ID': 'circ1',
                                      '_FROM_CROSSREF': [],
                                      'crossref': 'circ2',
                                      'date': '1911'},
                            'circ2': {'ENTRYTYPE': 'book',
                                      'ID': 'circ2',
                                      '_FROM_CROSSREF': [],
                                      'crossref': 'circ1',
                                      'date': '1911'}}
        parser = BibTexParser(add_missing_from_crossref=True)
        with self.assertLogs('bibtexparser.bibdatabase', level='ERROR') as cm:
            with open(input_file_path) as bibtex_file:
                bibtex_database = parser.parse_file(bibtex_file)
            self.assertIn("ERROR:bibtexparser.bibdatabase:Circular crossref dependency: circ1->circ2->circ1.", cm.output)
        self.assertDictEqual(bibtex_database.entries_dict, entries_expected)

    def test_crossref_missing_entries(self):
        input_file_path = 'bibtexparser/tests/data/crossref_missing_entries.bib'
        entries_expected = {'mcr': {'ENTRYTYPE': 'inbook',
                                    'ID': 'mcr',
                                    '_crossref': 'missing1',
                                    'author': 'Megan Mistrel',
                                    'crossref': 'missing1',
                                    'origdate': '1933',
                                    'title': 'Lumbering Lunatics'}}

        parser = BibTexParser(add_missing_from_crossref=True)
        with self.assertLogs('bibtexparser.bibdatabase', level='ERROR') as cm:
            with open(input_file_path) as bibtex_file:
                bibtex_database = parser.parse_file(bibtex_file)
            self.assertIn("ERROR:bibtexparser.bibdatabase:Crossref reference missing1 for mcr is missing.", cm.output)
        self.assertDictEqual(bibtex_database.entries_dict, entries_expected)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError as ex:
    print('[python-bibtexparser] setuptools not found. Falling back to distutils.core')
    from distutils.core import setup

with open('bibtexparser/__init__.py') as fh:
    for line in fh:
        if line.startswith('__version__'):
            version = line.strip().split()[-1][1:-1]
            break


def load_readme():
    with open("README.rst") as f:
        return f.read()


setup(
    name='bibtexparser',
    version=version,
    url="https://github.com/sciunto-org/python-bibtexparser",
    author="Francois Boulogne and other contributors",
    license="LGPLv3 or BSD",
    author_email="code@mweiss.ch",
    description="Bibtex parser for python 3",
    long_description_content_type="text/x-rst",
    long_description=load_readme(),
    packages=['bibtexparser'],
    install_requires=['pyparsing>=2.0.3'],
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import io
import unittest
from bibtexparser.bparser import BibTexParser


class TestHomogenizeFields(unittest.TestCase):

    def test_homogenize_default(self):
        with open('bibtexparser/tests/data/website.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read())
            entries = bib.get_entry_list()
            self.assertNotIn('url', entries[0])
            self.assertIn('link', entries[0])

    def test_homogenize_on(self):
        with open('bibtexparser/tests/data/website.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), homogenize_fields=True)
            entries = bib.get_entry_list()
            self.assertIn('url', entries[0])
            self.assertNotIn('link', entries[0])

    def test_homogenize_off(self):
        with open('bibtexparser/tests/data/website.bib', 'r') as bibfile:
            bib = BibTexParser(bibfile.read(), homogenize_fields=False)
            entries = bib.get_entry_list()
            self.assertNotIn('url', entries[0])
            self.assertIn('link', entries[0])

    def test_homogenizes_fields(self):
        self.maxDiff = None
        with io.open('bibtexparser/tests/data/article_homogenize.bib',
                     'r', encoding='utf-8') as bibfile:
            bib = BibTexParser(bibfile.read(), homogenize_fields=True)
            expected_dict = {
                'Cesar2013': {
                    'keyword': 'keyword1, keyword2',
                    'ENTRYTYPE': 'article',
                    'abstract': 'This is an abstract. This line should be '
                                'long enough to test\nmultilines... and with '
                                'a french érudit word',
                    'year': '2013',
                    'journal': 'Nice Journal',
                    'ID': 'Cesar2013',
                    'pages': '12-23',
                    'title': 'An amazing title',
                    'comments': 'A comment',
                    'author': 'Jean César',
                    'volume': '12',
                    'month': 'jan',
                    'url': "http://my.link/to-content",
                    'subject': "Some topic of interest",
                    'editor': "Edith Or",
                }
            }
            self.assertEqual(bib.get_entry_dict(), expected_dict)
